/**
 * EPG module
 */
tma.registerModule( epgpage );
epgpage.prototype = new tmaModule();
function epgpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgpage';

	this.pageGroup = 'epg';

	var self = this;

	/**
	 * list of EPG-data per channel (index: channelid)
	 *
	 * @type ChannelData[]
	 */
	var channelsData = {};

	/**
	 * Starttime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.starttime = 0;

	/**
	 * Endtime of the epg-timespan in milliseconds rounded to minutes
	 *
	 * @type {Number}
	 */
	this.endtime = 0;

	/**
	 * default query length in minutes
	 */
	this.timeSliceLength = 360; // 6h => 4 slices per day

	/**
	 * list of valid channels (no analogue channels)
	 */
	this.channels = [];

	/**
	 * pointers to channel containers (key: channelid)
	 */
	this.channelLine = {};

	/**
	 * list of conflicts
	 * @see this.calculateMarkerSlots
	 */
	this.conflicts = [];

	/**
	 * state of the EPG-grid
	 */
	var state = {
			curr_day:		'',
			scrollTop:		-1,
			scrollLeft:		-1,
			width:			0,
			height:			0,
			channelHeight:	0,
	};

	/**
	 * initial state (pseudo-)constant
	 */
	var EMPTY_STATE = $.extend({}, state);

	var $container = null;

	/* Settings */

	var prefs = {

			/**
			 * get the defaultfavourites setting
			 *
			 * @return {String}
			 */
			getFavGroup: function() {
					return tma.localSetting( 'defaultfavourites', '' );
			},

			/**
			 * set the defaultfavourites setting
			 *
			 * @param {String}	set		new fav. group setting
			 */
			setFavGroup: function(set) {
				tma.setLocalSetting( 'defaultfavourites', set );
			},

			/**
			 * get the scale setting
			 *
			 * @return {Number}
			 */
			getScale: function() {
				return parseInt(self.localSetting( 'scale', 7 ));
			},

			/**
			 * set the scale setting
			 *
			 * @param {Number}	set		new scale setting
			 */
			setScale: function(set) {
				self.setLocalSetting( 'scale', parseInt(set) );
			},

			/**
			 * get the displayMode setting
			 *
			 * @return {Number}		1: small
			 * 						2: middle
			 * 						3: large logo & name
			 */
			getDisplayMode: function() {
				return parseInt(self.localSetting( 'channel_display', 3 ));
			},

			/**
			 * set the displayMode setting
			 *
			 * @param {Number}	set		new displayMode setting
			 * 							1: small
			 * 							2: middle
			 * 							3: display logo & name
			 */
			setDisplayMode: function(set) {
				if ((parseInt(set) >= 1) && (parseInt(set) <= 3)) {
					self.setLocalSetting( 'channel_display', parseInt(set) );
				}
			},

			/**
			 * get the Primetime setting
			 *
			 * @return {String}
			 */
			getPrimetime: function() {
				return self.localSetting( 'primetime', '20:15' );
			},

			/**
			 * set the Primetime setting
			 *
			 * @param {String}	set		new Primetime
			 */
			setPrimetime: function(set) {
				if (set.match(/^\d{1,2}:\d{2}$/)) {
					self.setLocalSetting( 'primetime', set );
				}
			},

			/**
			 * get the number of visible days setting
			 *
			 * @return {Number}
			 */
			getNrOfDays: function() {
				return parseInt( self.localSetting( 'nrofdays', '7' ) );
			},

			/**
			 * get the genre-coloring setting (*on/off)
			 *
			 * @return {Boolean}
			 */
			getGenreColoring: function() {
				return parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1;
			},

			/**
			 * set the genre-coloring setting (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
			},

	};

	/**
	 * initialise module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// load genre list
			tma.apiEpgGenre().done(function() {

				$('<div id="debug">').appendTo('BODY')
				.css({
					position: 'absolute',
					top: 0,
					right: 0,
				});

				self.showEPG();

				// insert config menu item
				tma.call('addConfigItem', ['MainMenu', self]);

				self.insertConfig();

				// add shortcuts
				self.addShortcuts();

				$("#timestamp").hide(); // prevent vertical document scrollbar (vScroll in EPG-frame)

			});

		});

	};

	/**
	 * this page gets removed, so hide the modules configuration
	 */
	this.cleanup = function() {

		$('DIV#debug').remove();

		// remove mode-class
		$('DIV#content').removeClass('mode-edit-timer');

		// remove event handling
		$(tma).off('tmaTimerAdded.epgpage');
		$(tma).off('tmaTimerModified.epgpage');
		$(tma).off('tmaTimerDeleted.epgpage');
		$(tma).off('genre_colors_change.epgpage');
		$(window).off('resize.epgpage');

		// remove shortcuts
		this.removeShortcuts();

		// show app status
		$("#timestamp").show();

	};

	this.addShortcuts = function() {

		shortcut.add("Left"		, function() { $('.epg-nav BUTTON.ic-left').click(); });
		shortcut.add("Right"	, function() { $('.epg-nav BUTTON.ic-right').click(); });
		shortcut.add("Page_up"	, function() { $('.epg-nav BUTTON.epg-nav-day-next').click(); });
		shortcut.add("Page_down", function() { $('.epg-nav BUTTON.epg-nav-day-prev').click(); });
		shortcut.add("p"		, function() { $('.epg-nav BUTTON.epg-nav-primetime').click(); });
		shortcut.add("t"		, function() { $('.epg-nav BUTTON.ic-clock').click(); });
		shortcut.add("Down"		, function() { $('.epg-container').scrollTop(state.scrollTop + state.height); });
		shortcut.add("Up"		, function() { $('.epg-container').scrollTop(state.scrollTop - state.height); });

	};

	this.removeShortcuts = function() {

		shortcut.remove("Left");
		shortcut.remove("Right");
		shortcut.remove("Page_up");
		shortcut.remove("Page_down");
		shortcut.remove("p");
		shortcut.remove("t");
		shortcut.remove("Down");
		shortcut.remove("Up");

	};

	/**
	 * callback of timer modification drag-Objects
	 * @this {jkqDragDrop}
	 */
	this.onTimerModified = function(left, top, width) {

		/** @type jQueryObject */
		var $source = this.$source;
		$source.css({top: 0});
		var timer = $source.prop('timer');
		var start = Math.round( left		  / prefs.getScale()) * 60000 + self.starttime;
		var end   = Math.round((left + width) / prefs.getScale()) * 60000 + self.starttime;

		$source.data({
			start: start,
			duration: end - start,
		});

		$source.find('SPAN.epg-mark-fn-start').text(tma.date('H:i', start));
		$source.find('SPAN.epg-mark-fn-end'  ).text(tma.date('H:i', end  ));

		if (
				( tma.date('H:i', start) != tma.date('H:i', timer.starttime) ) ||
				( tma.date('H:i', end  ) != tma.date('H:i', timer.endtime  ) )
		) {
			$source.addClass('epg-mark-modified');
		} else {
			$source.removeClass('epg-mark-modified');
		}

	};

	/**
	 * add event handler to a timer marker
	 *
	 * @param {jQueryObject} $marker
	 */
	this.createTimerMarkerHandler = function($marker) {

		$marker
		.click(function() {

			// edit reservation

			// hope, that any module handles the 'editTimer' event
			tma.call( 'editTimer', [ $(this).prop('timer') ] ).done(function(popup) {
				self.removeShortcuts();
				$(popup).on('close', function() {
					self.addShortcuts();
				});
			});

		})

		.on('click', '.epg-mark-fn', function(e) {

			// set to active marker (display in front)
			if ( !$(e.target).is('BUTTON.ic-clock')) {
				// all elements, except BUTTON.ic-clock, should prevent opening
				// the timer edit window via $marker.click event
				e.stopPropagation();
			}

			$('.epg-mark-active').removeClass('epg-mark-active');
			$(this).parents('DIV.epg-mark').addClass('epg-mark-active');

		})

		.on('click', 'BUTTON.epg-mark-fn-jumptofirst', function(e) {

			// jump to the first occurance of this reservation
			e.stopPropagation();

			var day = parseInt($(this).attr('day'));

			var $container = $('DIV.epg-container');
			$container.scrollLeft($container.scrollLeft() - day * prefs.getScale() * 24*60 );

		})

		.on('click', 'BUTTON.ic-save', function(e) {

			// edit modified reservation or recording
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			/** @returns {TmaTimer} */
			var timer = $marker.prop('timer');

			var left = $marker.position().left;
			var width = $marker.outerWidth();
			var start = $marker.data().start;
			var end   = start + $marker.data().duration;
			var duration = parseInt( (end - start) / 60000 );

			// check, if any module handles the 'editTimer' event
			if (timer instanceof TmaTimer) {

				if (tma.fn.editTimer) {

					var modifications = {
						starttime:	tma.toPvrDate(start),
						duration:	duration,
						endtime:	tma.toPvrDate(end),
					};

					tma.fn.editTimer( timer, modifications ).done(function(popup) {
						self.removeShortcuts();
						$(popup).on('close', function() {
							self.addShortcuts();
						});
					});

				}

			} else {

				// current recording
				tma.apiFilesDuration(timer.slot, duration).done(function() {

					// adjust status.recording object
					timer.duration = duration;
					timer.endtime = tma.toPvrDate( (new Date(timer.starttime)).getTime() + duration * 60000 );

					$marker.attr('duration', duration);
					$marker.removeClass('epg-mark-modified');
					$marker.data('duration', duration);

					// recalc timer slots
					self.calculateMarkerSlots();

				});

			}

		})

		.on('click', 'BUTTON.ic-stop', function(e) {

			// stop current recording
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');

			new jkqDialog({
				title: self.i18.stoprec_dialog_title,
				text: self.i18.stoprec_dialog_text,
				autoSkipText: tma.i18.respond_auto_skip,
				autoSkipButton: 'epgpageStopRecording',
				buttons: {
					epgpageStopRecording:	tma.i18.respond_yes,
					jkqDialogCancel:		tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'epgpageStopRecording') {

					/** @returns {TmaTimer} */
					var timer = $marker.prop('timer');

					tma.stopRecording(timer.slot).done(function(data) {

						if (data.return_code == 1) {
							$marker.remove();
						}

					});

				}
			});

		})

		.on('click', 'BUTTON.ic-reset', function(e) {

			// reset modified reservation
			e.stopPropagation();

			var $marker = $(this).parents('DIV.epg-mark');
			var timer = $marker.prop('timer');

			var timer_start = new Date(timer.starttime).getTime();
			var timer_end	= new Date(timer.endtime  ).getTime();

			$marker.data({
				start: timer_start,
				duration: timer_end   - timer_start,
			});

			self.scaleMarker($marker);

			$marker.find('SPAN.epg-mark-fn-start').text(tma.date('H:i', timer_start));
			$marker.find('SPAN.epg-mark-fn-end'  ).text(tma.date('H:i', timer_end  ));

			$marker.removeClass('epg-mark-modified');

		})

		.on('click', 'BUTTON.ic-delete', function(e) {

			e.stopPropagation();

			// delete reservation
			/** @returns {TmaTimer} */
			var timer = $(this).parents('DIV.epg-mark').prop('timer');

			var filename;
			if (tma.config.utftoppy || !timer.filename_hex) {
				filename = timer.filename;
			} else {
				filename = tma.decodeHexname(timer.filename_hex);
			}

			new jkqDialog({
				title: tma.i18.timer.delete_confirm_caption,
				text:  tma.i18.timer.delete_confirm_text.format(timer.getFilename(true)),
				autoSkipText: tma.i18.respond_auto_skip,
				autoSkipButton: 'tmaDeleteTimer',
				buttons: {
					tmaDeleteTimer:	 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'tmaDeleteTimer') {

					tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

						new jkqAlert({
							title:	tma.i18.error.dialog_title,
							text:	tma.i18.error.timer.del + '<br><br>' + error
						}).confirm();

					});

				}

			});


		})

		// drag marker start
		.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-left', function(e) {

			e.stopPropagation();

			self.timerResizeL.start(e, $(this).parents('.epg-mark'));
			self.timerResizeL.onDragMove = self.onTimerModified;

		})

		// drag marker end
		.on('mousedown touchstart', 'BUTTON.epg-mark-fn-resize-right', function(e) {

			e.stopPropagation();

			self.timerResizeR.start(e, $(this).parents('.epg-mark'));
			self.timerResizeR.onDragMove = self.onTimerModified;

		})

		// drag marker
		.on('mousedown touchstart', 'DIV.epg-mark-fn-editable', function(e) {

			if ( !$(e.target).is('BUTTON') ) {

				e.stopPropagation();

				self.timerMove.start(e, $(this).parents('.epg-mark'), 500);
				self.timerMove.onDragMove = self.onTimerModified;

			}

		});

	};

	/**
	 * test, if timer is in visible time span
	 *
	 * @param {Number}	startTimestamp
	 * @param {Number}	endTimestamp
	 * @returns {Boolean}
	 */
	this.timerInRange = function(startTimestamp, endTimestamp) {

		var result =
		((startTimestamp	>= this.starttime) && (startTimestamp	<= this.endtime)) ||
		((endTimestamp		>= this.starttime) && (endTimestamp		<= this.endtime)) ||
		((startTimestamp	<= this.starttime) && (endTimestamp		>= this.endtime));

		return result;

	};

	/**
	 * creates a reservation bar for a timer, if it is in the EPG range
	 *
	 * @param {jQuery}		$bar		parent timer bar
	 * @param {TmaTimer}	timer		tma-timer object
	 * @returns {jqObject} or false
	 */
	this.createTimerMarker = function($bar, timer) {

		/**
		 * Valid days for recurring reservation
		 * default: every day is valid
		 */
		var recurringValidDays = [1,1,1,1,1,1,1];

		if (timer.reservationtype() == 2) {
			// weekend, only sunday and saturday are valid
			recurringValidDays = [1,0,0,0,0,0,1];
		} else if (timer.reservationtype() == 4) {
			// weekdays, only monday to friday are valid
			recurringValidDays = [0,1,1,1,1,1,0];
		}

		var timer_start = new Date(timer.starttime).getTime();
		var timer_end	= new Date(timer.endtime).getTime();

		var day = 0;
		var daysStep = (timer.reservationtype() == 3) ? 7 : 1; // weekly?
		while (timer_start < this.endtime) {

			var dayOfWeek = parseInt(tma.date('w', timer_start));
			if (
					(recurringValidDays[dayOfWeek]) &&
					this.timerInRange(timer_start, timer_end)
			) {

				var $marker = $( self.templates.timerbar.render({
					i18:		self.i18,
					type:		(timer instanceof TmaTimer) ? (timer.isrec ? 'rec' : 'play') : 'recslot',
					name:		timer.getFilename(true),
					starttime:	tma.date('H:i', timer.starttime),
					endtime:	tma.date('H:i', timer.endtime),
					day:		day,
					timestamp:	timer_start,
					duration:	timer_end - timer_start,
					timerid:	timer.id,
					istimer:	(timer instanceof TmaTimer)
				}) )
				//.attr('timerid', timer.id)
				.data({
					start: timer_start,
					duration: timer_end - timer_start,
				})
				.prop('timer', timer)

				.appendTo($bar);

				self.scaleMarker($marker);

				self.createTimerMarkerHandler($marker);

			}

			if (!timer.reservationtype()) {
				// one time
				break;
			}

			// next
			timer_start += daysStep*24*3600000;
			timer_end	+= daysStep*24*3600000;
			day			+= daysStep;

		}

	};

	/**
	 * returns a jQuery object with channel logo and name
	 *
	 * @param {TmaChannel}	channel		channel object
	 * @returns jqObject	(DIV.epg-head-channel)
	 */
	this.createChannelHead = function (channel) {

		var current = (tma.status.current_logoid  == channel.logoid);
		var recording = tma.status.getRecording(channel.logoid);

		return $(
				this.templates.channelhead.render({
					channel: channel,
					current: current,
					recording: recording
				})
		)
		.click( function() {
			// 'this' is the clicked element
			self.selectCurrentChannel( $(this) );
		});

	};

	/**
	 * returns a row container for channel
	 *
	 * @param {TmaChannel}	channel		channel object
	 * @returns jQueryObject	(DIV.epg-channel)
	 */
	this.createChannelRow = function (channel) {

		var $line = $(
			'<div class="epg-channel epg-bg" channelid="' + channel.logoid + '">' +
				'<div class="epg-channel-marker epg-bg"></div>' +
			'</div>').prop('channel', channel);
		var $prgs = $('<div class="epg-channel-prgs"></div>').appendTo($line);

		// timers
		/** @param {TmaTimer} timer */
		$.each(tma.timers.timers, function(ix, timer) {

			if (timer.logoid == channel.logoid) {

				self.createTimerMarker($line.find('DIV.epg-channel-marker'), timer);

			}

		});

		// recording slots
		var recording = tma.status.getRecording(channel.logoid);
		if ( recording ) {

			self.createTimerMarker($line.find('DIV.epg-channel-marker'), recording);

		}

		// show event details
		$prgs.on('click', 'DIV.epg-channel-prg', function(e) {

			var $prg = $(this);

			// check, if any module handles the 'showEvent' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showEvent) &&
					!$prg.hasClass('jk-mark')
			) {

				var event = $prg.prop('event');

				tma.fn.showEvent( event ).done(function(popup) {
					self.removeShortcuts();
					popup.$relatedPrg = $prg;
					$(popup).on('close', function() {
						this.$relatedPrg.removeClass('jk-mark');
						self.addShortcuts();
					});
				});

				$prg.addClass('jk-mark');

			}

		});

		return $line;

	};

	/**
	 * fills the timeline with the requested time range
	 */
	this.createTimeline = function () {

		var tl_start = new Date( parseInt(this.starttime / 3600000) * 3600000 ).getTime();
		var tl_hours = parseInt( (this.endtime - tl_start + 3599999) / 3600000);

		var hours = [];
		for ( var i = 0; i < tl_hours; i++, tl_start += 3600000 ) {
			hours.push({
				date: tma.date(this.i18.shortdate_fmt, tl_start),
				hour: tma.date('G' , tl_start )
			});
		}

		$('DIV.epg-container DIV.epg-timeline-frame DIV.epg-timeline').html(
				this.templates.timeline.render({
					hours: hours
				})
		);

		this.scaleTimeline();

	};

	this.calculateMarkerSlots = function() {

		// calculate async
		window.setTimeout( function() {
			self.asyncCalculateRecSlots();
		}, 1 );

	};

	this.asyncCalculateRecSlots = function() {

		/**
		 * create a sorted list of all reservation start- and endtimes
		 */
		var timerChanges = [];

		/**
		 * returns a timerChange object for a given timestamp
		 *
		 * @param {Number} timestamp
		 */
		function findTimerChange(timestamp) {
			for( var i=0; i < timerChanges.length; i++) {
				if (timestamp == timerChanges[i].timestamp) {
					return timerChanges[i];
				}
			}
			var timerChange = {
				timestamp: timestamp,
				starts:	[],
				ends:	[],
			};
			timerChanges.push(timerChange);
			return timerChange;
		}

		// cleanup
		$('.epg-timeline .epg-mark').remove();
		$('.epg-mark-denied').remove();
		$('.epg-mark-conflict').remove();

		// recording slots

		var recording, recSlot;
		for(recSlot = 0; recSlot < 4; recSlot++) {

			recording = tma.status.getRecording(recSlot);
			if ( recording ) {

				var rec_start	= new Date(recording.starttime).getTime();
				var rec_end		= new Date(recording.endtime).getTime();

				if ( this.timerInRange(rec_start, rec_end) ) {

					// start
					var timerChange = findTimerChange(rec_start);
					timerChange.starts.push({
						obj: recording,
						start: rec_start,
						end: rec_end,
						recurring: false,
					});

					// end
					timerChange = findTimerChange(rec_end);
					timerChange.ends.push({
						obj: recording,
						start: rec_start,
						end: rec_end,
						recurring: false,
					});

				}

			}
		}

		// timers

		/** @param {TmaTimer} timer */
		$.each(tma.timers.timers, function(ix, timer) {

			timer.markerSlot = {};

			/**
			 * Valid days for recurring reservation
			 * default: every day is valid
			 */
			var recurringValidDays = [1,1,1,1,1,1,1];

			if (timer.reservationtype() == 2) {
				// weekend, only sunday and saturday is valid
				recurringValidDays = [1,0,0,0,0,0,1];
			} else if (timer.reservationtype() == 4) {
				// weekdays, only monday to friday is valid
				recurringValidDays = [0,1,1,1,1,1,0];
			}

			var timer_start = new Date(timer.starttime).getTime();
			var timer_end	= new Date(timer.endtime).getTime();

			var day = 0;
			var daysStep = (timer.reservationtype() == 3) ? 7 : 1; // weekly?
			while (timer_start < self.endtime) {

				var dayOfWeek = parseInt(tma.date('w', timer_start));
				if (
						(recurringValidDays[dayOfWeek]) &&
						self.timerInRange(timer_start, timer_end)
				) {

					// start
					var timerChange = findTimerChange(timer_start);
					timerChange.starts.push({
						obj: timer,
						start: timer_start,
						end: timer_end,
						recurring: (day > 0),
					});

					// end
					timerChange = findTimerChange(timer_end);
					timerChange.ends.push({
						obj: timer,
						start: timer_start,
						end: timer_end,
						recurring: (day > 0),
					});

				}

				if (!timer.reservationtype()) {
					// one time
					break;
				}

				// next
				timer_start += daysStep*24*3600000;
				timer_end	+= daysStep*24*3600000;
				day			+= daysStep;

			}

		});


		// sort timerChanges by timestamps
		timerChanges.sort(function(tc1, tc2) {
			return tc1.timestamp - tc2.timestamp;
		});

		// allocate timers to virtual slots

		// initiate empty slots
		var slot;
		var virtualSlots = [];
		for (slot = 0; slot < tma.config.maxrecstreams; slot++) {
			virtualSlots.push( null );
		}
		var virtualTuners = [ [], [] ];
		var favGroup = tma.localSetting('defaultfavourites', '');
		var channels = tma.getChannels(favGroup);
		var inaccessibleChannels = {};
		this.conflicts = [];

		$.each(timerChanges, function(ix, timerChange) {

			// ends
			$.each(timerChange.ends, function(ix, changeData) {
				var slot = 0;
				for( slot = 0; slot < virtualSlots.length; slot++ ) {
					if (virtualSlots[slot] === changeData.obj) {
						// release slot
						virtualSlots[slot] = null;
						break;
					}
				}
				if (slot == virtualSlots.length) {
					// error: slot of timer not found
				}
				var tuner = 0, rec = 0;
				for ( tuner = 0; tuner < 2; tuner++) {
					for ( rec = 0; rec < virtualTuners[tuner].length; rec++) {
						if ( virtualTuners[tuner][rec] === changeData.obj ) {
							virtualTuners[tuner].splice(rec, 1);
							break;
						}
					}
				}

				$.each(self.conflicts, function(i, conflict) {
					if (
							(conflict.end === 0) &&
							(conflict.obj == changeData.obj)
					) {
						// conflict end
						conflict.end = timerChange.timestamp;
					}
				});

			});

			// release inaccessible channels
			if (
					(virtualTuners[0].length === 0) ||
					(virtualTuners[1].length === 0)
			) {
				/** @param {TmaChannel} channel */
				$.each(inaccessibleChannels, function(channelid, start) {

					if ( timerChange.timestamp - start > 0 ) {
						var data = {
							start: start,
							duration: timerChange.timestamp - start,
						};

						var $bar = $('<div class="epg-mark epg-bg epg-mark-denied" />');
						$bar.data(data);

						self.scaleMarker($bar);
//						$bar.appendTo('.epg-channel[channelid="' + channelid + '"] .epg-channel-marker');
						$bar.appendTo( $(self.channelLine[channelid]).find('.epg-channel-marker') );
					}

					delete inaccessibleChannels[channelid];

				});

			}
			// release conflicts
			if (timerChange.ends.length > 0 ) {
				$.each(self.conflicts, function(i, conflict) {
					if (conflict.end === 0) {
						// conflict not ended
						for( slot = 0; slot < virtualSlots.length; slot++ ) {
							if (!virtualSlots[slot]) {
								// we have a free slot
								if (
										(virtualTuners[0].length === 0) ||
										(virtualTuners[1].length === 0) ||
										(tma.channel( conflict.obj.logoid ).transponderindex ==
											tma.channel( virtualTuners[0][0].logoid ).transponderindex) ||
										(tma.channel( conflict.obj.logoid ).transponderindex ==
											tma.channel( virtualTuners[1][0].logoid ).transponderindex)
								) {
									// conflict end
									conflict.end = timerChange.timestamp;
								}

								break;
							}
						}
					}
				});
			}

			// starts
			$.each(timerChange.starts, function(ix, changeData) {

				var ch = tma.channel( changeData.obj.logoid );

				var tuner = -1;
				if (changeData.obj instanceof TmaStatusRecording) {
					// set tuner for recording objects
					tuner = changeData.obj.tuner;
				} else {
					// allocate tuner for timer objects
					if (
							(virtualTuners[0].length > 0) &&
							(tma.channel( changeData.obj.logoid )) &&
							(tma.channel( virtualTuners[0][0].logoid )) &&
							(tma.channel( changeData.obj.logoid ).transponderindex ==
								tma.channel( virtualTuners[0][0].logoid ).transponderindex)
					) {
						tuner = 0;
					} else if (
							(virtualTuners[1].length > 0) &&
							(tma.channel( changeData.obj.logoid )) &&
							(tma.channel( virtualTuners[1][0].logoid )) &&
							(tma.channel( changeData.obj.logoid ).transponderindex ==
								tma.channel( virtualTuners[1][0].logoid ).transponderindex)
					) {
						tuner = 1;
					} else if ( virtualTuners[0].length === 0 ) {
						tuner = 0;
					} else if ( virtualTuners[1].length === 0 ) {
						tuner = 1;
					}
				}

				slot = virtualSlots.length;
				if ( tuner > -1 ) {
					for( slot = 0; slot < virtualSlots.length; slot++ ) {
						if (!virtualSlots[slot]) {
							// occupy slot
							virtualSlots[slot] = changeData.obj;

							// occupy tuner
							virtualTuners[tuner].push( changeData.obj );

							// display bar in timeline

							var typeclass = 'epg-mark-';
							if (changeData.obj instanceof TmaTimer) {
								typeclass += (changeData.obj.isrec) ? 'rec' : 'play';
								typeclass += (changeData.recurring) ? '-ff' : '';
							} else if (changeData.obj instanceof TmaStatusRecording) {
								typeclass += 'recslot';
							}
							var filename = changeData.obj.getFilename(true);

							// create Timeline bar
							var $bar = $( self.templates.timelinebar.render({
										typeclass:	typeclass,
										vslot:		slot,
										channel:	tma.channel( changeData.obj.logoid ),
										filename:	filename
									}))
							.data({
								start: changeData.start,
								duration: changeData.end - changeData.start,
							})
							.appendTo('.epg-timeline');

							self.scaleMarker($bar);

							break;
						}
					}
				}
				if (slot == virtualSlots.length) {
					// error: no free slot or no free tuner available
					self.conflicts.push({
						start:	timerChange.timestamp,
						end:	0,
						obj:	changeData.obj,
					});
				}

			});

			// check for inaccessible channels
			if (
					(virtualTuners[0].length > 0) &&
					(virtualTuners[1].length > 0) &&
					(tma.channel( virtualTuners[0][0].logoid )) &&
					(tma.channel( virtualTuners[1][0].logoid ))
			) {
				var tpId0 = tma.channel( virtualTuners[0][0].logoid ).transponderindex;
				var tpId1 = tma.channel( virtualTuners[1][0].logoid ).transponderindex;

				// mark each inaccessible channel with start-timestamp
				/** @param {TmaChannel} channel */
				$.each(channels, function(ix, channel) {

					if (
							!inaccessibleChannels[channel.logoid] &&
							(channel.transponderindex != tpId0) &&
							(channel.transponderindex != tpId1)
					) {
						inaccessibleChannels[channel.logoid] = timerChange.timestamp;
					}

				});

			}

		});

		if (this.conflicts.length > 0) {

			$.each(this.conflicts, function(i, conflict) {

				var $bar = $('<div class="epg-mark epg-bg epg-mark-conflict" />');
				$bar.data({
					start: conflict.start,
					duration: conflict.end - conflict.start,
				});
				//$bar.appendTo('.epg-channel[channelid="' + conflict.obj.logoid + '"] .epg-channel-marker');
				$bar.appendTo( $(self.channelLine[conflict.obj.logoid]).find('.epg-channel-marker') );

				self.scaleMarker($bar);

			});

			$('.epg-show-conflicts').show();
			this.showConflicts(this.conflicts);

		} else {
			$('.epg-show-conflicts').hide();
		}

	};

	/**
	 * scale a timer or rec-slot marker
	 *
	 * @param {jQueryObject}	$marker
	 */
	this.scaleMarker = function($marker) {

		var data = $marker.data();
		var pos = Math.round( (data.start - this.starttime) * prefs.getScale() / 60000 );
		var w	= Math.round( (data.duration ) * prefs.getScale() / 60000 );

		$marker.css({
			left: pos,
			width: w,
		});

		if ($marker.hasClass('epg-mark-recslot')) {
			$marker.find('.epg-mark-fn-buttons').css('padding-left', -pos);
		}

	};

	this.getTimelineOffset = function() {

		var tl_start = new Date( parseInt(this.starttime / 3600000) * 3600000 ).getTime();

		return Math.round( (tl_start - this.starttime) * prefs.getScale() / 60000 ) - 1;

	};

	/**
	 * scales the timeline
	 */
	this.scaleTimeline = function () {

		var $timeline = $('DIV.epg-container DIV.epg-timeline-frame DIV.epg-timeline');
		$timeline.find('.epg-hour-marker').css({
			width: 60 * prefs.getScale(),
			left: this.getTimelineOffset()
		});
		$timeline.find('.epg-qtr-marker' ).css({ width: 15 * prefs.getScale() });

	};

	/**
	 * scales an event
	 *
	 * @param {jQueryObject} $event
	 */
	this.scaleEvent = function($event) {

		var pos = Math.round( ($event.data().start - this.starttime) * prefs.getScale() / 60000 );
		var w	= $event.data().duration * prefs.getScale();
		if (pos < 0) {
			w += pos;
			pos = 0;
		}

		$event.css({
			left: pos,
			width: w,
			minWidth: w,
		});

	};

	this.rescaleGrid = function() {

		this.scaleTimeline();

		$('DIV.epg-container .epg-mark').each(function() {
			self.scaleMarker( $(this) );
		});

		$('DIV.epg-container .epg-channel-prg').each(function() {
			self.scaleEvent( $(this) );
		});

	};

	/**
	 * process the result of the EGP query
	 *
	 * @param {TmaEpgEvents}	epg
	 */
	this.processEpgResult = function(epg) {

		if (epg.event) {

			var channelData = this.channelData(epg.channel.logoid);

			//var $prgs = $('DIV.epg-channel[channelid="' + epg.channel.logoid + '"] DIV.epg-channel-prgs');
			var $prgs = $(self.channelLine[epg.channel.logoid]).find('.epg-channel-prgs');

			if ($prgs.length) {

				/** @param {TmaEpgEvent} event */
				epg.eachEvent( function(ix, event) {

					if (!channelData.events[event.eventid]) {

						channelData.events[event.eventid] = true;

						var event_start = event.startDate().getTime();
						var event_end	= event.endDate().getTime();

						var current = 	(event_start < self.starttime) &&
						(event_end	 > self.starttime) ?
								' epg-prg-current' : '';

						if ( (event_end > self.starttime) && (event.duration > 0) ) {

							var $event = $( self.templates.event.render({
								event:		event,
								colouring:	prefs.getGenreColoring(),
								start:		tma.date('H:i', event.starttime_local),
								end:		tma.date('H:i', event.endtime_local),
								dur:		tma.formatDuration(event.duration),
							})
							).appendTo($prgs)
							.data({
								start: event_start,
								duration: event.duration,
							})
							.prop('event', event);

							self.scaleEvent($event);

						}

					}

				});

			}
		}

	};

	/**
	 * returns the channel data of a specific channel
	 *
	 * @param {String}	channelid
	 * @returns {ChannelData}
	 */
	this.channelData = function(channelid) {

		return channelsData[channelid];

	};

	/**
	 * load EPG events of a channels slice
	 *
	 * @param {TmaChannel}	channel
	 * @param {TimeSlice}	slice
	 */
	this.loadSlice = function(channel, slice) {

		var deferred = $.Deferred();

		if (!slice.state) {

			slice.state = 1; // loading
			tma.apiGetEpg(channel, slice.starttime, slice.endtime).done(function(epg) {
				slice.state = 2; // loaded
				slice.epg = epg;

				self.processEpgResult(epg);

				deferred.resolve();
			});

		} else if (slice.state == 2) {

			// data is available
			self.processEpgResult(slice.epg);
			deferred.resolve();

		}

		return deferred.promise();

	};

	/**
	 * load and display EPG data of visible time slices in visible channels
	 */
	this.updateEpg = function() {

		// update current day
		var curr_day = tma.date('Ymd', this.starttime + Math.round(state.scrollLeft * 60000 / prefs.getScale()) );
		if (curr_day != state.curr_day) {
			state.curr_day = curr_day;
			$('DIV.epg-nav-days BUTTON.epg-nav-day')
			.removeClass('epg-nav-current-day')
			.filter('BUTTON[date="' + curr_day + '"]')
			.addClass('epg-nav-current-day');
		}

		// check visible channels
		var fromChannelNr	= parseInt( state.scrollTop / state.channelHeight );
		var toChannelNr		= parseInt( (state.scrollTop + state.height) / state.channelHeight + 0.99);

		// check toChannelNr max
		toChannelNr	= Math.min(toChannelNr, this.channels.length - 1);

		// start timestamp in minutes
		var start = parseInt(this.starttime / 60000);

		var timeLeft	= this.starttime + parseInt( state.scrollLeft				 * 60000 / prefs.getScale());
		var timeRight	= this.starttime + parseInt((state.scrollLeft + state.width) * 60000 / prefs.getScale());

		// get visible time slices
		var fromSliceNr = parseInt( timeLeft  / (this.timeSliceLength * 60000) );
		var toSliceNr	= parseInt( timeRight / (this.timeSliceLength * 60000) );

		for( var chNr = fromChannelNr; chNr <= toChannelNr; chNr++ ) {

			/** @returns {TmaChannel} */
			var channel = this.channels[chNr];
			var channelData = self.channelData(channel.logoid);

			for(var sliceNr = fromSliceNr; sliceNr <= toSliceNr; sliceNr++) {

				var slice = channelData.slice(sliceNr);

				this.loadSlice(channel, slice);

			}

		}

	};

	/**
	 * renders the navigation bar
	 */
	this.createNavigationBar = function() {

		var date = this.starttime;
		var day = 0;
		var days = [];
		while( date < this.endtime) {

			days.push({
				day: day,
				date: tma.date('Ymd', date),
				date1: tma.date(this.i18.shortestdate_fmt, date),
				date2: tma.date(this.i18.shorterdate_fmt, date),
				date3: tma.date(this.i18.shortdate_fmt, date)
			});

			day++;
			date += 24*60*60000;
		}

		$(
				this.templates.navbar.render({
					i18: this.i18,
					prefs: prefs,
					days: days
				})
		).appendTo('DIV#content')
		.on('click', 'BUTTON.epg-nav-day', function() {

			var clickedDay = parseInt( $(this).attr('day') );
			var currentDay = parseInt( $('DIV.epg-nav-days BUTTON.epg-nav-current-day').attr('day') );

			var diff = (clickedDay - currentDay) * prefs.getScale() * 24*60;
			if ($container) $container.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.epg-nav-day-prev', function() {

			var diff = prefs.getScale() * 24*60;
			if ($container) $container.scrollLeft(state.scrollLeft - diff);

		}).on('click', 'BUTTON.epg-nav-day-next', function() {

			var diff = prefs.getScale() * 24*60;
			if ($container) $container.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.epg-nav-primetime', function() {

			var timeLeft = self.starttime + parseInt( state.scrollLeft * 60000 / prefs.getScale());
			var dateStrLeft = tma.date('Y/m/d H:i:s', timeLeft);
			var dateStrPrime = tma.date('Y/m/d ', timeLeft) + prefs.getPrimetime() + ':00';
			var dateLeft = new Date(dateStrLeft);
			var datePrime = new Date(dateStrPrime);

			var diff = parseInt(prefs.getScale() * (datePrime.getTime() - dateLeft.getTime()) / 60000);
			if ($container) $container.scrollLeft(state.scrollLeft + diff);

		}).on('click', 'BUTTON.ic-left', function() {

			if ($container) $container.scrollLeft(state.scrollLeft - state.width);

		}).on('click', 'BUTTON.ic-right', function() {

			if ($container) $container.scrollLeft(state.scrollLeft + state.width);

		}).on('click', 'BUTTON.ic-clock', function() {

			// toggle timermode
			$('DIV#content').toggleClass('mode-edit-timer');

		}).on('click', 'BUTTON.tma-test', function() {
			// run function to test
			self.asyncCalculateRecSlots();
		});

		$(window).on('resize.epgpage', function() {

			var h = $('.epg-nav > BUTTON').outerHeight(true) + 10;
			$('.epg-nav-days').attr('width-mode', 3);
			var widthMode = 3;
			while ((widthMode > 1) && ($('.epg-nav-days').height() > h)) {
				$('.epg-nav-days').attr('width-mode', --widthMode);
			}

		}).resize();

	};

	this.fillChannels = function(favGroup) {

		this.channels = [];
		$('DIV.epg-head'	).empty();
		$('DIV.epg-channels .epg-channel').remove();

		// create lines for each channel
		var channels = tma.getChannels(favGroup);
		/** @param {TmaChannel} channel */
		$.each(channels, function(ix, channel) {

			//DMC - Filter analogue channels, they have no EPG.
			if(channel.logoid.substring(0,14) != "ffffffffffffff") {
				self.channels.push(channel);
				$('DIV.epg-head'	).append( self.createChannelHead(channel) );
				var $channel = self.createChannelRow( channel);
				$('DIV.epg-channels').append( $channel );
				self.channelLine[channel.logoid] = $channel;

				if (!channelsData[channel.logoid]) {
					channelsData[channel.logoid] = new ChannelData(channel, self.starttime, self.timeSliceLength);
				} else {
					channelsData[channel.logoid].events = {};
				}
			}

		});

		state.channelHeight = $('.epg-channel:first').outerHeight();

	};

	/**
	 * load and display an EPG grid
	 *
	 * currently the EPG data is loaded for all channels of the selected favourites group (=> channels page)
	 */
	this.showEPG = function () {

		var favGroup = tma.localSetting('defaultfavourites', '');

		this.channels = [];
		$.extend(state, EMPTY_STATE);

		this.starttime = (tma.config.mock) ? new Date('2013/10/13 18:06:02') : new Date();
		this.starttime = Math.round(this.starttime / 60000) * 60000;
		var endtime = this.starttime + prefs.getNrOfDays() * 24*60*60000;
		var enddate = new Date( tma.date('Y/m/d', endtime) );
		this.endtime = enddate.getTime();

		var minutes = parseInt( (this.endtime - this.starttime) / 60000 );

		this.createNavigationBar();

		$container = $(
				this.templates.content.render({
					i18:			this.i18,
					width:			minutes * prefs.getScale() + 80,
					displayMode:	prefs.getDisplayMode(),
					ppm:			prefs.getScale(),
				})
		).appendTo('DIV#content');

		// iOS hack
		$container.on('dragstart drop', /*'DIV',*/ function(e) {
			e.preventDefault();
		});

		// move content with mouse
		if ( !Modernizr.touch ) {
			var $ch_container = $container.find('DIV.epg-channels');
			var dragndrop = new jkqDragDrop();
			dragndrop.onDragStart = function(e, drag_source) {
				this.drag_x_rel = - $container.scrollLeft()	- e.pageX;
				this.drag_y_rel = - $container.scrollTop()	- e.pageY;
			};
			dragndrop.dragmove = function(x, y) {
				$container.scrollLeft(-x - this.drag_x_rel);
				$container.scrollTop( -y - this.drag_y_rel);
			};

			// start drag
			$ch_container.on('mousedown', function(emd) {
				// move
				$ch_container.on('mousemove', function(emm) {
					if (	Math.sqrt(
							Math.pow(emm.pageX - emd.pageX, 2) +
							Math.pow(emm.pageY - emd.pageY, 2)
						) > 5
					) {
						$ch_container.off('mousemove');
						dragndrop.start(emd, $ch_container);
					}
				});
			}).on('mouseup', function() {
				$ch_container.off('mousemove');
			});
		}

		var deferred = $.Deferred();

		// first get status to update current channel and rec channels
		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			// then get timers to set reservation-bars
			tma.apiGetTimers().done(function(timers) {

				// setup the timeline
				self.createTimeline();

				// create lines for each channel
				self.fillChannels( favGroup );

				self.calculateMarkerSlots();

				// keep channel on left border
				// and timeline on top (scroll in EPG-container)
				$container.scroll(function() {

					state.scrollTop = $container.scrollTop();
					$container.find('DIV.epg-timeline').parent().css('top', state.scrollTop);

					state.scrollLeft = $container.scrollLeft();
					$container.find('DIV.epg-head').css('left', state.scrollLeft);

					self.updateEpg();

				}).scroll();

				// adjust vertical size (vScroll in EPG-frame)
				$(window).on('resize.epgpage', function() {

					var offs = $container.offset();
					var maxH = /*$(document).scrollTop() +*/ $(window).height() - offs.top - 15;
					$container.css('max-height', maxH);

					state.scrollTop = $container.scrollTop();
					state.scrollLeft = $container.scrollLeft();

					state.width = $container.width() - $('DIV.epg-head').outerWidth();
					state.height = $container.height() - $('DIV.epg-timeline-frame').outerHeight();

					self.updateEpg();

				}).resize();

				deferred.resolve();

			}).fail(function() {
				deferred.reject();
			});

		}).fail(function() {
			deferred.reject();
		});

		// handle added timer
		$(tma).on('tmaTimerAdded.epgpage', function(e, timer) {

			// recalc timer slots
			self.calculateMarkerSlots();
			// add reservation
			//var $bar = $('DIV.epg-channel[channelid="' + timer.logoid + '"] DIV.epg-channel-marker');
			var $bar = $(self.channelLine[timer.logoid]).find('.epg-channel-marker');
			self.createTimerMarker($bar, timer);

		});

		// handle modified timer
		$(tma).on('tmaTimerModified.epgpage', function(e, timer) {

			// remove prev reservetion
			$('DIV.epg-mark[timerid="' + timer.id + '"]').remove();
			// recalc timer slots
			self.calculateMarkerSlots();
			// add modified reservation
			//var $bar = $('DIV.epg-channel[channelid="' + timer.logoid + '"] DIV.epg-channel-marker');
			var $bar = $(self.channelLine[timer.logoid]).find('.epg-channel-marker');
			self.createTimerMarker($bar, timer);

		});

		// handle deleted timer
		$(tma).on('tmaTimerDeleted.epgpage', function(e, timer) {

			// recalc timer slots
			self.calculateMarkerSlots();
			// remove prev reservetion
			$('DIV.epg-mark[timerid="' + timer.id + '"]').remove();

		});

		// show conflict list
		$('BUTTON.epg-show-conflicts').click( function() {

			self.showConflicts(self.conflicts);

		});

		// handle click on timline timer bar
		$('.epg-timeline').on('click', '.epg-mark', function() {

			var channelid = $(this).attr('channelid');
			var $head = $('.epg-head-channel[channelid="' + channelid + '"]');
			var pos = $head.position();
			if (pos) {
				if ($container) {
					//$container.scrollTop(pos.top - parseInt(state.height / 3));
					$container.animate({
						scrollTop: pos.top - parseInt(state.height / 3)
					}, 1000);
				}
			}

		});

		return deferred.promise();

	};

	/**
	 * selects a channel and marks the element with css class 'current-channel'
	 *
	 * @param {jQueryObject} $channel	channel element with attribute channelid
	 */
	this.selectCurrentChannel = function ($channel) {

		var channel = tma.channel( $channel.attr('channelid') );
		if ( channel ) {
			tma.apiChannelSet( channel.chtype, channel.chnum ).done(function() {
				// set new current channel mark
				$('.current-channel').removeClass('current-channel');
				$channel.addClass('current-channel');
			}).fail(function(xhr, errorType, error) {

				if (errorType == 'tma-error') {
					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.channels.set
					}).confirm();
				}

			});
		}

	};

	var popup = new jkqWindow();

	/**
	 * show conflict list in a window
	 *
	 * @param {Array}	conflicts
	 */
	this.showConflicts = function(conflicts) {

		popup.show();
		popup.setTitle( this.i18.conflicts_caption );

		popup.container.find('DIV.jkqWindowContent')
		.html( self.templates.conflicts.render({
				conflicts: conflicts,
				filenameNoExt: function() { return this.obj.getFilename(true); },
				formatStarttime: function() {  return tma.date( self.i18.timerdate_fmt, this.obj.starttime ); },
		}) )

		.on('click', '.epg-conflict', function(e) {
			e.stopPropagation();

			var start = parseInt($(this).attr('start'));
			var channelid = $(this).attr('channelid');
			var $head = $('.epg-head-channel[channelid="' + channelid + '"]');
			var pos = $head.position();
			if (pos) {
				//if ($container) $container.scrollTop(pos.top - parseInt(state.height / 3));

				pos.left = Math.round( (start - self.starttime) * prefs.getScale() / 60000 );
				//if ($container) $container.scrollLeft(pos.left - parseInt(state.width / 3));
				if ($container) {
					$container.animate({
						scrollTop: pos.top - parseInt(state.height / 3),
						scrollLeft: pos.left - parseInt(state.width / 3)
					}, 1000);

				}
			}

		});

		popup.adjustPos({
			hAlign: 'left',
			effect: 'slide-ltr'
		});

	};

	function onGenreColorChanged() {

		//console.log('epgpage.onGenreColorChanged');
		// genre color configuration has changed
		$('.epg-prg-time[genre-value]').each(function() {
			var genre = tma.genres.genre( parseInt($(this).attr('genre-value')) );
			if (genre) {
				$(this).attr('style', genre.style( prefs.getGenreColoring() ));
			}
		});

	}

	this.insertConfig = function () {

		$('DIV#config').append($(this.templates.config.render({
			config:			this.i18.config,
			favgroups:		tma.i18.favgroups,
			scale:			'{0}%'.format(prefs.getScale() * 10),
		})));

		var $colorSettings = tma.fn.getModule('genrecolours').createConfig(
				prefs.getGenreColoring(),
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
					onGenreColorChanged();
				}
		);

		$(tma).on('genre_colors_change.epgpage', onGenreColorChanged);

		$('DIV#config').append($colorSettings);

		/**
		 * fav group
		 */

		var $fav = $('DIV#config SELECT[name="favourites"]');
		/** @param {TmaFavouritesGroup} group */
		tma.favourites.eachGroup( function(nr, group) {

			if (group.count !== 0) {
				//Only list groups with members
				$fav.append(
						'<option value="{name}">{getName} ({count})</option>'.format(group)
				);
			}

		});
		$fav.val( prefs.getFavGroup() );
		$fav.change(function() {

			var favGroup = $(this).val();
			prefs.setFavGroup( favGroup );
			self.fillChannels( favGroup );
			self.updateEpg();

		});

		/**
		 * scale
		 */

		$('DIV#config DIV.cfg-scale *.less').click(function() {

			var scale = prefs.getScale() - 1;
			if (scale >= 1) {
				prefs.setScale(scale);
				$('DIV#config DIV.cfg-scale .value').text('{0}%'.format(scale * 10));
				$('.epg-channels').attr( 'ppm', scale );
				self.rescaleGrid();
				self.updateEpg();
			}

		});
		$('DIV#config DIV.cfg-scale *.more').click(function() {

			var scale = prefs.getScale() + 1;
			prefs.setScale(scale);
			$('DIV#config DIV.cfg-scale .value').text('{0}%'.format(scale * 10));
			$('.epg-channels').attr( 'ppm', scale );
			self.rescaleGrid();

		});

		$('DIV#config INPUT[name="logotext"]')
		.click(function() {

			var val = $('DIV#config INPUT[name="logotext"]:checked').val();
			prefs.setDisplayMode( val );
			$('.epg-channels').attr( 'display-mode', val );
			state.channelHeight = $('.epg-channel:first').outerHeight();
			self.updateEpg();

		})
		.filter('[value="' +  prefs.getDisplayMode() + '"]' ).prop('checked', true);

		/**
		 * Primetime
		 */
		$('DIV#config INPUT[name="primetime"]').blur(function() {
			var val = $(this).val();
			if (val.match(/^\d{1,2}:\d{2}$/)) {
				prefs.setPrimetime( val );
				$('.epg-nav BUTTON.epg-nav-primetime SPAN').text(val);
			} else {
				$(this).val(prefs.getPrimetime());
			}
		});

	};

	// init reservation resize

	// drag left end, to expand reservation
	this.timerResizeL = new jkqResizeLeft();

	// drag right end, to expand reservation
	this.timerResizeR = new jkqResizeRight();

	// move reservation
	this.timerMove = new jkqDragDrop();

}

function TimeSlice(nr) {

	/**
	 * slice number relative to timstamp 0
	 */
	this.nr = nr;

	/**
	 * query state:
	 * 	0: empty
	 * 	1: loading
	 *  2: ready
	 */
	this.state = 0; // empty

	/**
	 * slice start in PVR date format
	 */
	this.starttime = '';

	/**
	 * slice end in PVR date format
	 */
	this.endtime = '';

	/**
	 * api EPG response for this slice
	 *
	 * @returns {TmaEpgEvents}
	 */
	this.epg = null;

}

/**
 * @param {Object}	channel
 * @param {Date}	starttime	timestamp of the grids starttime
 * @param {int}		sliceLength	minutes per slice
 */
function ChannelData(channel, starttime, sliceLength) {

	this.channel = channel;

	var slices = {};

	this.events = {};

	/**
	 * returns a time slice
	 *
	 * @param {Number}	channelid
	 * @returns {TimeSlice}
	 */
	this.slice = function(sliceNr) {

		if (!slices[sliceNr]) {

			slices[sliceNr] = new TimeSlice(sliceNr);
			var start = sliceNr * sliceLength * 60000;
			var end = start + sliceLength * 60000;
			slices[sliceNr].starttime = tma.toPvrDate( start );
			slices[sliceNr].endtime = tma.toPvrDate(end);

		}

		return slices[sliceNr];

	};

}
