/**
 * Plugin: Test for click-/touchable osd elements
 */

// type definitions

function TRGBA() {
	this.r = 0;
	this.g = 0;
	this.b = 0;
	this.a = 0;
}

function TScanVector() {
	this.x = 0;
	this.y = 0;
	this.len = 0;
}

/**
 * Scan Vector right
 * @param len
 * @returns {TScanVector}
 */
function DirRight(len) {
	return { x: 1, y: 0, len: len };
}

/**
 * Scan Vector left
 * @param len
 * @returns {TScanVector}
 */
function DirLeft(len) {
	return { x: -1, y: 0, len: len };
}

/**
 * Scan Vector down
 * @param len
 * @returns {TScanVector}
 */
function DirDown(len) {
	return { x: 0, y: 1, len: len };
}

/**
 * Scan Vector up
 * @param len
 * @returns {TScanVector}
 */
function DirUp(len) {
	return { x: 0, y: -1, len: len };
}

function TPoint() {
	this.x = 0;
	this.y = 0;
}

function TRegion() {
	this.valid = false;
	this.left =	0;
	this.top = 0;
	this.right = 0;
	this.bottom = 0;
	this.width = 0;
	this.height = 0;
	this.color = '';
	this.selected = false;

	/**
	 * create a jQuery DIV representing the region
	 * 
	 * @param {String}	asClass	css class orButton, orDialog, orRegion 
	 * @returns {jQueryObject} 
	 */
	this.createNode = function(asClass) {
		return $('<div class="' + asClass + '" />').css({
			left: this.left,
			top: Math.round( this.top * hScale),
			width: this.width,
			height: Math.round( this.height * hScale),
		}).addClass( (this.selected ? 'orSelected' : '' ));
	};
	
}

/**
 * creates a button
 * 
 * @param {TRegion}	region
 */
function TButton(region) {
	
	this.r = region;
	this.$node = region.createNode('orButton');
	this.$node.data('TButton', this);
	this.$node.click(buttonClick);
	this.index = 0;
	/** @returns {TLayer} */
	this.layer = null;
	
}

/**
 * creates a layer
 * 
 * @param {String}	type	list, dialog
 * @param {TRegion}	region
 * @returns {TButton}
 */
function TLayer(type, region) {
	this.type = type || '';
	this.r = $.extend(new TRegion(), region);
	this.buttons = [];
	this.$node = $('<div class="orLayer" />').css({
		left: 0,
		top: 0,
		width: 720,
		height: Math.round( 576 * hScale),
	}).appendTo('#tvscreen');
	
	/**
	 * creates a button and adds it to the list
	 * 
	 * @param {TRegion}	region
	 * @returns {TButton}
	 */
	this.addButton = function(region) {
		
		var button = new TButton(region);
		button.index = this.buttons.length;
		button.layer = this;
		this.$node.append(button.$node);
		this.buttons.push(button);
		return button;
		
	};
	
	/**
	 * get the index of the selected button
	 * 
	 * @returns {Number}
	 */
	this.selectedIndex = function() {
		
		var i, button;
		for( i = 0; i < this.buttons.length; i++ ) {
			button = this.buttons[i];
			if (button.r.selected) {
				return i;
			}
		}
		
		return -1;
		
	};
	
	/**
	 * tests if an updated block is in this region
	 */
	this.updated = function() {
		
		var left 	= this.r.left	>> 4;
		var right	= this.r.right	>> 4;
		var top		= this.r.top	>> 4;
		var bottom	= this.r.bottom	>> 4;
		var x, y;
		
		var result = false;	
		for ( y = top; y <= bottom; y++) {
			for ( x = left; x <= right; x++) {
				
				if (tmsr.updatedMacroblocks[x][y]) {
					result = true;
					break;
				}
			}
		}
		
		return result;
		
	};
	
}

TLayers = function() {
	
	var list = [];
	
	this.length = 0;
	
	/**
	 * creates a layer and adds it to the list
	 * 
	 * @param {String}	type	list, dialog
	 * @param {TRegion}	region
	 * @returns {TLayer}
	 */
	this.addLayer = function(type, region) {
		
		var recent = this.recentLayer();
		if (recent) {
			recent.$node.addClass('or-inactive');
		}
		layer = new TLayer(type, region);
		list.push(layer);
		this[this.length] = layer;
		this.length = list.length;
		return layer;
		
	};
	
	/**
	 * returns the last layer of the list
	 * 
	 * @returns {TLayer} or false, if list is empty
	 */
	this.recentLayer = function() {
		
		if (list.length) {
			return list[list.length - 1];
		}
		return false;
		
	};
	
	/**
	 * returns the last layer of the list and removes it
	 * 
	 * @returns {TLayer} or false, if list is empty
	 */
	this.pop = function() {
		
		var layer = list.pop();
		this.length = list.length;
		delete this[this.length];
		if (layer) {
			layer.$node.remove();
			var recent = this.recentLayer();
			if (recent) {
				recent.$node.removeClass('or-inactive');
			}
			return layer;
		}
		return false;
		
	};
	
	/**
	 * deletes all layers in list
	 */
	this.clear = function() {
		
		while (layers.length) {
			layers.pop();
		}
		
	};
	
};
var layers = new TLayers();

var contexts;
var $seqLog;
var doSeqLog = true;
var hScale = 1;

/* Array of bytes to base64 string decoding */

function base64DecToArr (sBase64, nBlocksSize) {

	function b64ToUint6 (nChr) {

		return nChr > 64 && nChr < 91 ?
				nChr - 65
				: nChr > 96 && nChr < 123 ?
						nChr - 71
						: nChr > 47 && nChr < 58 ?
								nChr + 4
								: nChr === 43 ?
										62
										: nChr === 47 ?
												63
												:
													0;

	}

	var
	sB64Enc = sBase64.replace(/[^A-Za-z0-9\+\/]/g, ""), nInLen = sB64Enc.length,
	nOutLen = nBlocksSize ? Math.ceil((nInLen * 3 + 1 >> 2) / nBlocksSize) * nBlocksSize : nInLen * 3 + 1 >> 2,
	ta = new ArrayBuffer(nOutLen);
	taBytes = new Uint8Array(ta);

	for (var nMod3, nMod4, nUint24 = 0, nOutIdx = 0, nInIdx = 0; nInIdx < nInLen; nInIdx++) {
		nMod4 = nInIdx & 3;
		nUint24 |= b64ToUint6(sB64Enc.charCodeAt(nInIdx)) << 18 - 6 * nMod4;
		if (nMod4 === 3 || nInLen - nInIdx === 1) {
			for (nMod3 = 0; nMod3 < 3 && nOutIdx < nOutLen; nMod3++, nOutIdx++) {
				taBytes[nOutIdx] = nUint24 >>> (16 >>> nMod3 & 24) & 255;
			}
			nUint24 = 0;

		}
	}

	return ta;
}

function base64EncFromArr(arrayBuffer) {
	var base64    = '';
	var encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

	var bytes         = new Uint8Array(arrayBuffer);
	var byteLength    = bytes.byteLength;
	var byteRemainder = byteLength % 3;
	var mainLength    = byteLength - byteRemainder;

	var a, b, c, d;
	var chunk;

	// Main loop deals with bytes in chunks of 3
	for (var i = 0; i < mainLength; i = i + 3) {
		// Combine the three bytes into a single integer
		chunk = (bytes[i] << 16) | (bytes[i + 1] << 8) | bytes[i + 2];

		// Use bitmasks to extract 6-bit segments from the triplet
		a = (chunk & 16515072) >> 18; // 16515072 = (2^6 - 1) << 18
		b = (chunk & 258048)   >> 12; // 258048   = (2^6 - 1) << 12
		c = (chunk & 4032)     >>  6; // 4032     = (2^6 - 1) << 6
		d = chunk & 63;               // 63       = 2^6 - 1

		// Convert the raw binary segments to the appropriate ASCII encoding
		base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d];
	}

	// Deal with the remaining bytes and padding
	if (byteRemainder == 1) {
		chunk = bytes[mainLength];

		a = (chunk & 252) >> 2; // 252 = (2^6 - 1) << 2

		// Set the 4 least significant bits to zero
		b = (chunk & 3)   << 4; // 3   = 2^2 - 1

		base64 += encodings[a] + encodings[b] + '==';
	} else if (byteRemainder == 2) {
		chunk = (bytes[mainLength] << 8) | bytes[mainLength + 1];

		a = (chunk & 64512) >> 10; // 64512 = (2^6 - 1) << 10
		b = (chunk & 1008)  >>  4; // 1008  = (2^6 - 1) << 4

		// Set the 2 least significant bits to zero
		c = (chunk & 15)    <<  2; // 15    = 2^4 - 1

		base64 += encodings[a] + encodings[b] + encodings[c] + '=';
	}

	return base64;
}

/**
 * decodes a test sequence
 * 
 * @param {String} text
 */
function decodeTestSequence(text) {

	var lines = text.replace(/\r/, '').split('\n');
	
	if (!contexts) {
		contexts = create_canvas();
	}

	var screens = [];
	var screen = [];
	var line, firstChar, i;
	for ( i = 0; i < lines.length; i++) {
		line = lines[i];
		if (line.length > 0) {
			firstChar = line.substr(0,1);
			if (firstChar == '-') {
				screens.push(screen);
				screen = [];
			} else if (firstChar != ' ') {
				screen.push(line);
			}
		}
		
	}
	
	var $div = $('#test');
	$div.find('BUTTON[idx]').remove();
	
	var buttonClick = function() {
		lines = screens[ $(this).attr('idx') ];
		tmsr.OsdFrameComplete = true;
		for ( var l = 0; l < lines.length; l++) {
			tmsr.decodeMsgData(base64DecToArr(lines[l]), contexts.osd, contexts.tv);
		}
		$(tmsr).triggerHandler('ws_message', [tmsr.MID_OSDframeComplete]);
	};
	
	for ( i = 0; i < screens.length; i++) {
		
		$('<button idx="' + i + '">scr ' + i + '</button>')
		.appendTo($div)
		.click(buttonClick);
		
	}
	
	layers.clear();
	$('#seq-log > LI').remove();

}

$(function() {

	// insert a base 64 input area
	$('TEXTAREA#log')
	.after('<p>Recorded OSD macroblocks (base64 data). Copy to text editor and save for offline tests.<br><textarea id="base64" /></p>')
	.val('')
	.attr('spellcheck', 'false');
	$('TEXTAREA#base64').css({
		width: 720,
		height: 200
	});
	
	$(
			'<div id="test">' +
				'<span>Offline test:</span>' +
				'<select name="test">' +
					'<option value="">[select test sequence]</option>' +
					'<option value="option-delete-left-exit.txt">QuickPlay delete</option>' +
					'<option value="option-play-delete-right-exit-stop.txt">FastSkip delete</option>' +
					'<option value="tmsarchive-delete.txt">TMSArchive delete</option>' +
					'<option value="filer-down-down-exit.txt">Filer down-down-exit</option>' +
					'<option value="guide-5-nav-exit.txt">SmartEPG page 5 nav-exit</option>' +
					'<option value="quickplay-up-to-top.txt">QuickPlay up to top</option>' +
					'<option value="smartepg-page5.txt">SmartEPG page 5</option>' +
					'<option value="smartepg-page6.txt">SmartEPG page 6</option>' +
				'</select>' +
				//'<button>load</button>' + 
			'</div>'
	)
	.insertAfter('#options')
	.find('SELECT').change(function() {
		
		var file = $(this).val();
		if (file) {

			$.ajax({
				url : "test/" + file,
				dataType: "text",
			}).done(function (data) {
				decodeTestSequence(data);
			});			

		}		

	});
	
	$seqLog = $('<ul id="seq-log" />').appendTo('BODY')
	/** @param {jQueryEvent} e */
	.on('click', 'LI', function(e) {
		e.stopPropagation();
		$(this).children('UL').slideToggle();
	});
	
	$('<button>remove regions</button>').appendTo('#options')
	.click(function() {
		$('#tvscreen > DIV').remove();
	});

	$('#options SELECT').change(function() {
		var resolution = $(this).val().split('x');
		hScale = parseInt(resolution[1]) / 576;
		layers.clear();
	}).change();
	
	$(
			'<input type="checkbox" id="tt-record-comm" style="vertical-align: middle; margin: 0 5px;"> ' +
			'<label for="tt-record-comm" style="vertical-align: middle;">record communication</label>'
	)
	.appendTo('#options');
	
	$('<button>upd. blocks</button>')
	.appendTo('#options')
	.click(function() {
		var existingBlocks = $('#tvscreen .orBlock');
		if ( existingBlocks.length ) {
			existingBlocks.remove();
		} else {
			showUpdatedBlocks();
		}
	});
	
	$('<button>run tests</button>')
	.insertAfter('#refreshlog')
	.click(runTests);

});

/**
 * log text in OR functions
 * 
 * @param {jQueryObject}	$parent
 * @param {String}			text
 * @returns {jQueryObject}	UL of this log entry
 */
function seqLog($parent, text) {
	if (!doSeqLog) return false;
	return $('<li><div>' + text + '</div><ul /></li>').appendTo($parent).find('UL');
}

/**
 * log region in OR functions
 * 
 * @param {jQueryObject}	$parent
 * @param {TRegion}			r
 * @returns {jQueryObject}	UL of this log entry
 */
function seqLogRegion($parent, r) {
	if (!doSeqLog) return false;
	return seqLog($parent, '('+r.left+','+r.top+'),('+r.right+','+r.bottom+') '+r.width+'x'+r.height+' found: '+r.valid+' #'+r.color.toString(16) );
}

/**
 * calculate updated region
 *
 * @returns {TRegion}
 */
function getUpdatedRegion() {

	var r = new TRegion();
	r.right = 44;
	r.top = 35;
	var x, y;
	
	for ( y = 0; y < 36; y++) {
		for ( x = 0; x < 45; x++) {
			
			if (tmsr.updatedMacroblocks[x][y]) {
				if (x < r.left)	r.left = x;
				if (x > r.right)	r.right = x;
				if (y < r.top)		r.top = y;
				if (y > r.bottom)	r.bottom = y;
			}
		}
	}
	
	if ( (r.left < r.right) && (r.top < r.bottom) ) {
		r.valid	= true;
		r.left	= (r.left	<< 4);
		r.top	= (r.top	<< 4);
		r.right	= (r.right	<< 4) + 15;
		r.bottom= (r.bottom	<< 4) + 15;
		r.width	= r.right - r.left + 1;
		r.height= r.bottom - r.top + 1;
	}

	return r;

}

/**
 * calculate updated centred region
 *
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}
 */
function getUpdatedCentredRegion($log) {

	var r = new TRegion();
	var x, y, vl, vr, vt, vb;
	
	// first test a 5x4 field at screen center
	var allUpd = true;	
	for ( y = 16; y <= 19; y++) {
		for ( x = 20; x <= 24; x++) {
			
			if (!tmsr.updatedMacroblocks[x][y]) {
				allUpd = false;
				break;
			}
		}
	}
	
	if (allUpd) {
		r.left = 20;
		r.right = 24;
		r.top = 16;
		r.bottom = 19;
		r.valid = true;
		
		vl = true;
		vr = true;
		vt = true;
		vb = true;
		
		while (vl || vr || vt || vb) {
		
			if (vl) {
				// extend left
				x = r.left - 1;
				for ( y = r.top; y <= r.bottom; y++) {
					if (!tmsr.updatedMacroblocks[x][y]) {
						vl = false;
						break;
					}
				}
				if (vl) {
					r.left = x;
					// consider left screen border
					if (r.left === 0) vl = false;
				}
			}
			
			if (vr) {
				// extend right
				x = r.right + 1;
				for ( y = r.top; y <= r.bottom; y++) {
					if (!tmsr.updatedMacroblocks[x][y]) {
						vr = false;
						break;
					}
				}
				if (vr) {
					r.right = x;
					// consider right screen border
					if (r.right === 44) vr = false;
				}
			}
			
			if (vt) {
				// extend top
				y = r.top - 1;
				for ( x = r.left; x <= r.right; x++) {
					if (!tmsr.updatedMacroblocks[x][y]) {
						vt = false;
						break;
					}
				}
				if (vt) {
					r.top = y;
					// consider top screen border
					if (r.top === 0) vt = false;
				}
			}
			
			if (vb) {
				// extend bottom
				y = r.bottom + 1;
				for ( x = r.left; x <= r.right; x++) {
					if (!tmsr.updatedMacroblocks[x][y]) {
						vb = false;
						break;
					}
				}
				if (vb) {
					r.bottom = y;
					// consider bottom screen border
					if (r.bottom === 35) vb = false;
				}
			}
		
		}
				
	}
	
	if ( r.valid ) {
		r.left	= (r.left	<< 4);
		r.top	= (r.top	<< 4);
		r.right	= (r.right	<< 4) + 15;
		r.bottom= (r.bottom	<< 4) + 15;
		r.width	= r.right - r.left + 1;
		r.height= r.bottom - r.top + 1;

		if ($log) {
			seqLog( $log , 'getUpdatedCentredRegion:');
			seqLogRegion( $log, r );
		}
	}

	return r;

}

/**
 * count all updated blocks
 */
function updatedBlocksCount() {

	var x, y;
	var result = 0;
	
	for ( y = 0; y < 36; y++) {
		for ( x = 0; x < 45; x++) {
			
			if (tmsr.updatedMacroblocks[x][y]) {
				result++;
			}
		}
	}

	seqLog($seqLog, 'updatedBlocksCount: ' + result);
	
	return result;

}

/**
 * visualise all updated blocks
 */
function showUpdatedBlocks() {

	var x, y;
	
	for ( y = 0; y < 36; y++) {
		for ( x = 0; x < 45; x++) {
			
			if (tmsr.updatedMacroblocks[x][y]) {
				
				$('<div class="orBlock" />').css({
					left: x << 4,
					top: Math.round( (y << 4) * hScale ),
					width: 16,
					height: Math.round( 16 * hScale),
				}).appendTo('#tvscreen');
			}
		}
	}

}

/**
 * get the color at a specific coordinate
 * 
 * @param {TPoint}			point
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRGBA}	color  
 */
function getColor(point, $log) {
	
	var $subLog = seqLog($log, 'getColor at ('+point.x+','+point.y+')' );
	
	// get color
	var bx = point.x >> 4;
	var by = point.y >> 4;
	var block = tmsr.macroblocks[bx][by];
	seqLog($subLog, 'block('+bx+','+by+')' );

	var i = (((point.y & 0xf) << 4) + (point.x & 0xf)) << 2;
	var result = new TRGBA();
	result.r = block.data[i++];
	result.g = block.data[i++];
	result.b = block.data[i++];
	result.a = block.data[i];
	
	seqLog($subLog, 'color: rgba('+result.r+', '+result.g+', '+result.b+', '+result.a+')' );
	
	return result; 
	
}

/**
 * get the color at a specific coordinate
 * 
 * @param {TPoint}			point
 * @param {jQueryObject}	$log		parent log element
 * @returns {Number}	color  
 */
function getColorDw(point, $log) {
	
	var $subLog = seqLog($log, 'getColorDw at ('+point.x+','+point.y+')' );
	
	// get color
	var bx = point.x >> 4;
	var by = point.y >> 4;
	var block = tmsr.macroblocks[bx][by];
	var cols = new Uint32Array(block.data.buffer);
	seqLog($subLog, 'block('+bx+','+by+')' );

	var i = ((point.y & 0xf) << 4) + (point.x & 0xf);
	
	seqLog($subLog, 'color: #' + cols[i].toString(16) );
	
	return cols[i]; // & 0x7fffffff;
	
}

/**
 * follow a color in scanDir
 * 
 * @param {TPoint}			start		starting point
 * @param {TScanVector}		scanDir
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}  
 */
function followColor(start, scanDir, $log) {
	
	var i;
	var bx = 0xffff;
	var by = 0xffff;
	var block, cols, dw;
	var r = new TRegion();
	var x = start.x;
	var y = start.y;

	var $subLog = seqLog($log, 'followColor: start= ('+start.x+','+start.y+')' );
	
	// get color
	bx = x >> 4;
	by = y >> 4;
	block = tmsr.macroblocks[bx][by];
	cols = new Uint32Array(block.data.buffer);
	seqLog($subLog, 'block('+bx+','+by+')' );

	i = ((y & 0xf) << 4) + (x & 0xf);
	r.color = cols[i]; // & 0x7fffffff;
		
	seqLog($subLog, 'color: #' + r.color.toString(16) );
		
	r.valid = true;
	r.left = x;
	r.top = y;
	r.right = x;
	r.bottom = y;
	
	var len = scanDir.len;
	while(len > 1) {
		
		len--;
		x += scanDir.x;
		y += scanDir.y;
		
		if ( ((x >> 4) != bx) || ((y >> 4) != by)) {
			bx = x >> 4;
			by = y >> 4;
			block = tmsr.macroblocks[bx][by];
			cols = new Uint32Array(block.data.buffer);
			seqLog($subLog, 'block('+bx+','+by+')' );
		}
			
		i = ((y & 0xf) << 4) + (x & 0xf);
		dw = cols[i]; // & 0x7fffffff;
		//colOk = ((dw & borderMask) == borderCol);
		colOk = (dw == r.color);
			
		seqLog($subLog, '('+x+','+y+'): #' + dw.toString(16) + ' ' + colOk );

		if (colOk) {
			if (scanDir.x > 0) {
				r.right++;
			} else if (scanDir.x < 0) {
				r.left--;
			} else if (scanDir.y > 0) {
				r.bottom++;
			} else if (scanDir.y < 0) {
				r.top--;
			}
		} else {
			break;
		}

	}

	r.width  = r.right  + 1 - r.left;
	r.height = r.bottom + 1 - r.top;

	seqLogRegion( $log, r );
	
	return r;
	
}

/**
 * scan to the first color change in scanDir
 * 
 * @param {TPoint}			start		starting point
 * @param {TScanVector}		scanDir
 * @param {Number}			color
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}  
 */
function scanToColor(start, scanDir, color, $log) {
	
	var i;
	var bx = 0xffff;
	var by = 0xffff;
	var block, cols;
	var r = new TRegion();
	var x = start.x;
	var y = start.y;

	var $subLog = seqLog($log, 'search color: start= ('+start.x+','+start.y+')' );
	
	// find line
	var len = scanDir.len;
	while ( len > 0) {
		
		if ( ((x >> 4) != bx) || ((y >> 4) != by)) {
			bx = x >> 4;
			by = y >> 4;
			block = tmsr.macroblocks[bx][by];
			cols = new Uint32Array(block.data.buffer);
			seqLog($subLog, 'block('+bx+','+by+')' );
		}
		
		i = ((y & 0xf) << 4) + (x & 0xf);
		dw = cols[i]; // & 0x7fffffff;
		
		colOk = (dw == color);
		
		seqLog($subLog, '('+x+','+y+'): #' + dw.toString(16) + ' ' + colOk );
		
		if (colOk) {
			r.valid = true;
			r.left = x;
			r.top = y;
			r.right = x;
			r.bottom = y;
			r.color = dw;
			break;
		}
		
		len--;
		if (len > 0) {
			x += scanDir.x;
			y += scanDir.y;
		}
		
	}
	
	if (r.valid) {
		r.width  = r.right  + 1 - r.left;
		r.height = r.bottom + 1 - r.top;
	}

	seqLogRegion( $log, r );
	
	return r;
	
}

/**
 * scan to the first color change in scan direction
 * and follow the new color in line direction
 * 
 * @param {TPoint}			start		starting point
 * @param {TScanVector}		scanDir		scan direction
 * @param {TScanVector}		lineDir		line direction
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}  
 */
function scanForLine(start, scanDir, lineDir, $log) {

	var r = new TRegion();
	
	var $subLog = seqLog($log, 'search line: start= ('+start.x+','+start.y+')' );

	var r1 = followColor(start, scanDir, $subLog);
	
	
	if ( Math.max(r1.width, r1.height) < scanDir.len ) {
		// color change detected
		var lineStart = {
				x: scanDir.x + ( (scanDir.x >= 0) ? r1.right : r1.left),
				y: scanDir.y + ( (scanDir.y >= 0) ? r1.bottom : r1.top),
		};

		r = followColor(lineStart, lineDir, $subLog);
	}

	seqLogRegion( $log, r );
	
	return r;
	
}

/**
 * intersects two regions
 * 
 * @param {TRegion} 		r1
 * @param {TRegion} 		r2
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}
 */
function regionAnd(r1, r2, $log) {
	
	var r = new TRegion();
	
	r.left   = Math.max(r1.left  , r2.left  );
	r.top    = Math.max(r1.top   , r2.top   );
	r.right  = Math.min(r1.right , r2.right );
	r.bottom = Math.min(r1.bottom, r2.bottom);
	
	r.valid = r1.valid && r2.valid && (r.left <= r.right) && (r.top <= r.bottom);
	if (r.valid) {
		r.width  = r.right  + 1 - r.left;
		r.height = r.bottom + 1 - r.top;
	}

	var $sublog = seqLog($log, 'regionAnd:');
	seqLogRegion( $sublog, r );
	
	return r;
	
}

/**
 * find button in dialog region
 * 
 * @param {TRegion}			dr			dialog region
 * @param {TPOINT}			start		scan start point
 * @param {jQueryObject}	$log		parent log element
 * @returns {TRegion}
 */
function findButton(dr, start, $log) {
	
	var bgCol;
	var r = new TRegion();
	var r1, r2;
	var $subLog;
	var scanHeight = 50;
	
	$log = seqLog($log, 'find button');
	
	// get backgroud color
	$subLog = seqLog($log, 'get backgroud color:');
	bgCol = getColorDw( start, $subLog );
	
	// find button bottom
	while(start.x < dr.right - 1 ) {

		$subLog = seqLog($log, 'find button bottom: start=('+start.x+','+start.y+')');
		
		r1 = followColor( start, DirUp(scanHeight), $subLog );

		if (r1.height < scanHeight) {
			
			r.bottom = r1.top - 1;
			seqLog($log, 'button-bottom: '+r.bottom );
					
			// find button top
			$subLog = seqLog($log, 'find button top');
			
			r2 = scanToColor( {x: start.x, y: r.bottom}, DirUp(44), bgCol, $subLog );
			
			if (r2.valid) {
				
				r.top = r2.bottom + 1;
 				r.height = r.bottom - r.top + 1;
				seqLog( $log, 'button-top: '+r.top );
						
				break;
			}
		}
		
		seqLog($subLog, 'not found');
		r1.valid = false;
		start.x += 50;
	} 
	
	if (r1.valid) {
		
		// find button left
		$subLog = seqLog($log, 'find button left');
		start = {x: r1.left, y: r.top};
		
		r2 = followColor( start, DirLeft(start.x - dr.left), $subLog );
		
		if (r2.valid) {
			
			r.left = r2.left;
			seqLog( $log, 'button-left: '+r.left );
			
			// find button right
			$subLog = seqLog($log, 'find button right');
 			
			r2 = followColor( start, DirRight(dr.right - start.x), $subLog );
 			
 			if (r2.valid) {
 				
 				r.right = r2.right;
 				r.width = r.right - r.left + 1;
 				r.color = r2.color;
 				seqLog( $log, 'button-right: '+r.right );
		
 				if ( r.width > 50 && r.height > 20 ) {
  
 					r.valid = true;
 	 				
 	 				// test if selected 
 					r.selected = isSelected({x: r.left, y: r.top}, $log);
					
 				}
			
 			} // END button right
	
		} // END button left
		
	} // END button top/botton found

	seqLogRegion( $log, r );
	
	return r;
	
}

/**
 * do region dimensions possibly fit to a dialog
 * 
 * @param {TRegion}	r
 * @returns {Boolean}
 */
function isPossibleDialogRegion(r) {
	
	return ( 
			(r.left   >= 144) && (r.left   <= 192) &&
			(r.right  >= 512) && (r.right  <= 560) &&
			(r.top    >= 186) && (r.top    <= 224) &&
			(r.bottom >= 336) && (r.bottom <= 384) 					
	);

}

/**
 * do region dimensions possibly fit to a list
 * 
 * @param {TRegion}	r
 * @returns {Boolean}
 */
function isPossibleListRegion(r) {
	
	return ((r.width * r.height) >= 104448) || (updatedBlocksCount() > 1000);
//	return ( 
//			(r.left   <   32) &&
//			(r.right  >= 688) &&
//			(r.top    <   96) &&
//			(r.bottom >= 544) 					
//	);

}

/**
 * test if selected (compute deviation to gray)
 * 
 * @param {TPoint}			point
 * @param {jQueryObject}	$log		parent log element
 * @returns {Boolean}
 */
function isSelected(point, $log) {
	
	var $subLog = seqLog($log, 'test if selected');
	
	var c = getColor(point, $subLog);

	// compute deviation to gray
	var cAvg = parseInt((c.r + c.g + c.b) / 3);
	var devGray = Math.abs(c.r - cAvg) + Math.abs(c.g - cAvg) + Math.abs(c.b - cAvg);
	
	seqLog($log, 'selected: ' + (devGray > 15) + ' (devGray=' + devGray + ')');
	
	return (devGray > 15);

}

/**
 * test if selected (if bg is gradient, the color changes before bottom)
 * 
 * @param {TPoint}		start
 * @param {TScanVector}	scanDir
 * @param {jQueryObject}	$log		parent log element
 * @returns {Boolean}
 */
//function testSelectedGradient(start, scanDir, $log) {
//	
//	var $subLog = seqLog($subLog, 'test if selected');
//
//	var r1 = followColor( start, scanDir, $subLog ); 				
//	
//	return (r1.height < scanDir.len);
//
//}

/**
 * find dialog frame in region
 * 
 * @param {TRegion}	region
 * @returns {TRegion}
 */
function findDialogFrame(region) {
	
	var rh, rv, ri, start;
	
	var r = new TRegion();
	
	var $log = seqLog($seqLog, 'findDialog');
	
	// scan left-top block for left-top corner
	var $subLog = seqLog($log, 'scan for left-top corner');
	start = {x: region.left + 15, y: region.top + 15};
	
	rh = scanForLine( start, DirUp(16), DirLeft(16), $subLog );
	rv = scanForLine( start, DirLeft(16), DirUp(16), $subLog );
	ri = regionAnd(rh, rv, $subLog);
	
	if (ri.valid) {
		
		r.left = ri.left;
		r.top  = ri.top;

		seqLog( $log, 'top-left('+r.left+','+r.top+')' );
		
		// scan left-bottom block for bottom border
		$subLog = seqLog($log, 'scan for left-bottom corner');
		start = {x: region.left + 15, y: region.bottom - 15};
		
		rh = scanForLine( start, DirDown(16), DirLeft(16), $subLog );
		rv = scanForLine( start, DirLeft(16), DirDown(16), $subLog );
		ri = regionAnd(rh, rv, $subLog);
		
		if (ri.valid) {
			
			r.bottom = ri.bottom;

			seqLog($log, 'left-bottom('+r.left+','+r.bottom+')' );
						
			// scan right-top block for right border
			$subLog = seqLog($log, 'scan for right-top corner');
			start = {x: region.right -15, y: region.top + 15};
			
			rh = scanForLine( start, DirUp(16), DirRight(16), $subLog );
			rv = scanForLine( start, DirRight(16), DirUp(16), $subLog );
			ri = regionAnd(rh, rv, $subLog);
			
			if (ri.valid) {
				
				r.right  = ri.right;
				r.width  = r.right  + 1 - r.left;
				r.height = r.bottom + 1 - r.top;

				seqLog($log, 'top-right('+r.right+','+r.top+')' );

				r.valid = true;
				
			}
		}
	}

	seqLogRegion( $log, r );

	return r;
	
}

function splitButton(br, br1, br2) {
	
	br1.left = br.left;
	br1.top = br.top;
	br1.width = parseInt(br.width / 2);
	br1.height = br.height;
	br1.right = br1.left + br1.width - 1;
	br1.bottom = br.bottom;
	br1.selected = false;
	
	br2.left = br.right - br1.width + 1;
	br2.top = br.top;
	br2.width = br1.width;
	br2.height = br.height;
	br2.right = br.right;
	br2.bottom = br.bottom;
	br2.selected = false;

}

/**
 * find buttons in dialog layer
 * 
 * @param {TLayer} layer
 * @param {jQueryObject}	$log		parent log element
 */
function findButtons(layer, $log) {
	
	layer.buttons = [];
	$(layer.$node).find('.orButton').remove();

	var button;
	var start = {x: layer.r.left + 50, y: layer.r.bottom - 2};
	var br = findButton(layer.r, start, $log);
	while (br.valid) {

		if (br.width < 180) {

			layer.addButton(br);

		} else {

			// two buttons

			var br1 = new TRegion();
			var br2 = new TRegion();

			splitButton(br, br1, br2);

			layer.addButton(br1);
			layer.addButton(br2);

		}

		if (layer.r.right - br.right < 100) break;

		start.x = br.right + 25;
		start.y = br.bottom + 1;

		br = findButton(layer.r, start, $log);
	}

}

/**
 * find first list item
 * 
 * @param {TPoint} start
 * @returns {TRegion}
 */
function findFirstListItem(startPoint) {
	
	var r = new TRegion();
	var r1, r2, r3, len;
	var start = $.extend({}, startPoint);

	var $subLog, $subLog2;
	$subLog = seqLog( $seqLog, 'find first list item: start=('+start.x+','+start.y+')' );

	var bgCol = getColorDw(start, $subLog);
	if (bgCol) {

		var scanBottom = Math.min(576, startPoint.y + 100);
		while (start.y < scanBottom) {

			len = scanBottom - start.y;
			$subLog2 = seqLog($subLog, 'scan down to color change');
			r1 = followColor(start, DirDown(len), $subLog2);
			if (r1.height < len) {
				var scanStart = {
						x: r1.left,
						y: r1.bottom + 1
				};
				$subLog2 = seqLog($subLog, 'get width of color change');
				r2 = followColor(scanStart, DirLeft(scanStart.x), $subLog2);
				r3 = followColor(scanStart, DirRight(720 - scanStart.x), $subLog2);
				if (r2.width + r3.width > 250) {
					r.left = r2.left;
					r.top = r2.top;
					r.right = r3.right;
					r.width = r2.width + r3.width;
					$subLog2 = seqLog( $subLog, 'list top=('+r.left+','+r.top+') width: ' + r.width );

					r2 = scanToColor( {x: r.left , y: r.top}, DirDown(44), bgCol, $subLog2 );
					r3 = scanToColor( {x: r.right, y: r.top}, DirDown(44), bgCol, $subLog2 );

					if (r2.valid && r3.valid && (r2.height == r3.height) ) {

						r.bottom = r2.top - 1;
						r.height = r.bottom - r.top + 1;

						if (r.height >= 22) {
							r.valid = true;

							// test if selected 
							r.selected = isSelected({x: r.left, y: r.top}, $subLog);
						}
					}

					break;
				}

				len = scanBottom - scanStart.y;
				$subLog2 = seqLog($subLog, 'scan down to next bg-color');
				r1 = scanToColor(scanStart, DirDown(len), bgCol, $subLog2);
				if (!r1.valid) break;
				start.y = r1.top;

			} else {
				break;
			}
		}

	}

	seqLogRegion( $subLog, r );

	return r;
	
}

/**
 * find next list item
 * 
 * @param {TRegion} prevItem
 * @returns {TRegion}
 */
function findNextListItem(prevItem) {
	
	var r = new TRegion();
	r.left = prevItem.left;
	r.right = prevItem.right;
	r.width = prevItem.width;

	var r1, r2, r3, len;
	var start = {
			x: prevItem.left,
			y: prevItem.bottom + 1
	};

	var $subLog, $subLog2;
	$subLog = seqLog( $seqLog, 'find next list item: start=('+start.x+','+start.y+')' );
	
	var bgCol = getColorDw(start, $subLog);

	len = Math.min(8, 576 - start.y);
	$subLog2 = seqLog($subLog, 'scan down to color change');
	r1 = followColor(start, DirDown(len), $subLog2);
	if (r1.height < len) {

		r.top = r1.bottom + 1;

		r2 = scanToColor( {x: r.left , y: r.top}, DirDown(44), bgCol, $subLog2 );
		r3 = scanToColor( {x: r.right, y: r.top}, DirDown(44), bgCol, $subLog2 );

		if (r2.valid && r3.valid && (r2.height == r3.height) ) {

			r.bottom = r2.top - 1;
			r.height = r.bottom - r.top + 1;

			if (r.height >= 22) {
				r.valid = true;

				// test if selected 
				r.selected = isSelected({x: r.left, y: r.top}, $subLog);

			}
		}
			
	}

	seqLogRegion( $subLog, r );

	return r;
	
}

var commRec = '';

function buttonClick() {
	
	var keyCode, repeats;
	/** @returns {TButton} */
	var button = $(this).data('TButton');
	
	var diff = button.index - button.layer.selectedIndex();
	
	if (diff === 0) {
		
		// ok
		keyCode = parseInt($('.keyOk').attr('keyCode'));
		//alert('ok');
		repeats = 1;
		
	} else {

		repeats = Math.abs(diff);
		if (button.layer.type == 'dialog') {

			if (diff < 0) {

				// left
				keyCode = parseInt($('.keyLeft').attr('keyCode'));
				//alert('left('+repeats+')');

			} else {

				// right
				keyCode = parseInt($('.keyRight').attr('keyCode'));
				//alert('right('+repeats+')');

			}

		} else if (button.layer.type == 'list') {

			if (diff < 0) {

				// up
				keyCode = parseInt($('.keyUp').attr('keyCode'));
				//alert('up('+repeats+')');

			} else {

				// down
				keyCode = parseInt($('.keyDown').attr('keyCode'));
				//alert('down('+repeats+')');

			}

		}

	}
	
	var	keyTimer;
	tmsr.keyDown(keyCode);
	tmsr.keyUp();
	console.log('repeats: '+repeats);
	repeats--;
	if (repeats > 0) {
		keyTimer = window.setInterval(
				function () {
					// repeatedly send key press
					tmsr.keyDown(keyCode);
					tmsr.keyUp();
					console.log('repeats: '+repeats);
					repeats--;
					if (repeats === 0) {
						window.clearInterval(keyTimer);
					}
				},
				200
		);
	}

}

$(tmsr).on('ws_message', function (e, msgNr) {
	
	var layer;
	
	if ($('#tt-record-comm').is(':checked')) {
		$('#base64').val( function(i, text) {
			return text + commRec + "-Msg: OSD Frame complete\n";
		});
		commRec = '';
	}
	

	if (msgNr == tmsr.MID_OSDframeComplete) {

		var r = getUpdatedCentredRegion();
		if (r.valid) {

			var $rLog = seqLog($seqLog, 'centred Region update');
			seqLogRegion($rLog, r);
			
			if ( isPossibleDialogRegion(r) ) {

				// is it a dialog box
				var dr = findDialogFrame(r);
				if (dr.valid) {
	
					layer = layers.addLayer('dialog', dr);
					// display dialog block region
					layer.$node.append( r.createNode("orRegion") );
						
					// display dialog frame
					layer.$node.append( dr.createNode("orDialog") );		

					findButtons(layer, $seqLog);

				} else {
					
					// is recent layer a dialog?
					layer = layers.recentLayer();
					if (layer && (layer.type == 'dialog')) {

						// recent layer is a dialog => remove
						layers.pop();

					}
				}
				
			} else if ( isPossibleListRegion(r) ) {

				// is recent layer a list?
				layer = layers.recentLayer();
				if (layer && (layer.type == 'list')) {

					// recent layer is a list => remove
					layers.pop();

				}
				
				var lr = findFirstListItem({x: 360, y: 40});
				if (lr.valid) {
					layer = layers.addLayer('list', r);
					while (lr.valid) {

						// display item
						layer.addButton(lr);

						lr = findNextListItem(lr);
					}
				}
			
			} else {
				
				// set invalid to force selection change scan 
				r.valid = false;
				
			}
		
		} 

		if (!r.valid) {
			
			// updated region is not h an v centred
			// or no dialog/list was detected
			// recent layer
			layer = layers.recentLayer();
			if ( 
					layer && 
					layer.buttons.length &&
					layer.updated() 
			) {
					
				if ( layer.type == 'dialog' ) {
					
					// => rescan dialog buttons
					findButtons(layer, $seqLog);
					
				} else if ( layer.type == 'list' ) {
					
					var lrf = findFirstListItem({x: 360, y: 40});
					if ( !lrf.valid ) {
						// remove layer
						layers.pop();
					} else if ( lrf.width != layer.buttons[0].r.width ) {
						// new layer => remove buttons
						layer.$node.find('.orButton').remove();
						layer.buttons = [];
						while (lrf.valid) {

							// display item
							layer.addButton(lrf);

							lrf = findNextListItem(lrf);
						}
					} else {
						// re-test item selection
						var $log = seqLog($seqLog, 'test list item selection');
						var i, button;
						for (i = 0; i < layer.buttons.length; i++) {
							button = layer.buttons[i];
							button.r.selected = isSelected({x: button.r.left, y: button.r.top}, $log);
							if (button.r.selected) {
								button.$node.addClass('orSelected');
							} else {
								button.$node.removeClass('orSelected');
							}
						}
					}
				}
			}

		}

	}

});

$(tmsr).on('ws_open', function() {

	doSeqLog = false;

});

$(tmsr).on('ws_close', function (e, data) {

    $('#tvscreen DIV').remove();
    doSeqLog = true;

});

$(tmsr).on('ws_osdblock', function (e, data) {

	if ($('#tt-record-comm').is(':checked')) {
		commRec += base64EncFromArr(data) + "\n";
	}

});

function testLog(text) {
	$('#log').val( function(i, prev) {
		return prev + text + "\n";
	});
}

function test(caption, expected, actual) {
	var result = (expected === actual);
	var s = '  ' + caption + ' should be ' + expected;
	while (s.length < 55) s += ' ';
	s += (result ? ' [ok]' : ' [FAILED]');
	testLog(s);
	if (!result) {
		testLog('          actual value: ' + actual);
	}
	return result;
}

function test1() {
	
	// QuickPlay with delete dialog

	return $.ajax({
		url : "test/option-delete-left-exit.txt",
		dataType: "text",
	}).done(function (data) {
	
		testLog('\ntest QuickPlay with delete dialog');
		
		decodeTestSequence(data);
		
		testLog(' Show QuickPlay');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[9].r.selected'	, true	, layers[0].buttons[9].r.selected);

		testLog(' Show dialog');
		$('#test BUTTON[idx]').eq(1).click();
		test('layers.length'					, 2		, layers.length);
		test('layers[1].buttons.length'			, 2		, layers[1].buttons.length);
		test('layers[1].buttons[0].r.selected'	, false	, layers[1].buttons[0].r.selected);
		test('layers[1].buttons[1].r.selected'	, true	, layers[1].buttons[1].r.selected);

		testLog(' Switch buttons');
		$('#test BUTTON[idx]').eq(2).click();
		test('layers[1].buttons[0].r.selected'	, true	, layers[1].buttons[0].r.selected);
		test('layers[1].buttons[1].r.selected'	, false	, layers[1].buttons[1].r.selected);
		
		testLog(' Exit dialog');
		$('#test BUTTON[idx]').eq(3).click();
		test('layers.length'					, 1		, layers.length);
		
	});		
	
}

function test2() {
	
	// FastSkip delete dialog

	return $.ajax({
		url : "test/option-play-delete-right-exit-stop.txt",
		dataType: "text",
	}).done(function (data) {
		
		testLog('\ntest FastSkip delete dialog');
		
		decodeTestSequence(data);
		$('#test BUTTON[idx]').eq(11).click();
		
		test('layers.length'					, 1		, layers.length);
		test('layers[0].buttons.length'			, 3		, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, true	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);
		testLog(' Next button');
		$('#test BUTTON[idx]').eq(12).click();
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, true	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);
		testLog(' Next button');
		$('#test BUTTON[idx]').eq(13).click();
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, true	, layers[0].buttons[2].r.selected);
		testLog(' Exit dialog');
		$('#test BUTTON[idx]').eq(14).click();
		test('layers.length'					, 0		, layers.length);
		
	});		
	
}

function test3() {
	
	// TMSArchive with delete dialog

	return $.ajax({
		url : "test/tmsarchive-delete.txt",
		dataType: "text",
	}).done(function (data) {
	
		testLog('\ntest TMSArchive with delete dialog');
		
		decodeTestSequence(data);
		
		
		testLog(' Show TMSArchive');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[13].r.selected'	, true	, layers[0].buttons[13].r.selected);

		testLog(' Show dialog');
		$('#test BUTTON[idx]').eq(12).click();
		test('layers.length'					, 2		, layers.length);
		test('layers[1].buttons.length'			, 2		, layers[1].buttons.length);
		test('layers[1].buttons[0].r.selected'	, false	, layers[1].buttons[0].r.selected);
		test('layers[1].buttons[1].r.selected'	, true	, layers[1].buttons[1].r.selected);
		
		testLog(' Switch buttons');
		$('#test BUTTON[idx]').eq(13).click();
		test('layers[1].buttons[0].r.selected'	, true	, layers[1].buttons[0].r.selected);
		test('layers[1].buttons[1].r.selected'	, false	, layers[1].buttons[1].r.selected);
		
		testLog(' Exit dialog');
		$('#test BUTTON[idx]').eq(15).click();
		test('layers.length'					, 1		, layers.length);
		
		testLog(' Exit TMSArchive');
		$('#test BUTTON[idx]').eq(37).click();
		test('layers.length'					, 0		, layers.length);
		
	});		
	
}

function test4() {
	
	// Filer

	return $.ajax({
		url : "test/filer-down-down-exit.txt",
		dataType: "text",
	}).done(function (data) {
	
		testLog('\ntest Filer');
		
		decodeTestSequence(data);
		
		
		testLog(' Show Filer (1)');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					, 0		, layers.length);
		
		testLog(' Show Filer (2)');
		$('#test BUTTON[idx]').eq(1).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);

		testLog(' Show Filer (3)');
		$('#test BUTTON[idx]').eq(2).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, true	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);
		
		testLog(' Switch down (1)');
		$('#test BUTTON[idx]').eq(3).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, true	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);
		
		testLog(' Switch down (2)');
		$('#test BUTTON[idx]').eq(4).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, true	, layers[0].buttons[2].r.selected);
		
		testLog(' Exit Filer');
		$('#test BUTTON[idx]').eq(5).click();
		test('layers.length'					, 0		, layers.length);
		
	});		
	
}

function test5() {

	// SmartEPG with page 5 and navigation

	return $.ajax({
		url : "test/guide-5-nav-exit.txt",
		dataType: "text",
	}).done(function (data) {

		testLog('\ntest SmartEPG with page 5 and navigation');

		decodeTestSequence(data);


		testLog(' Show SmartEPG');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[3].r.selected'	, false	, layers[0].buttons[3].r.selected);
		test('layers[0].buttons[5].r.selected'	, true	, layers[0].buttons[5].r.selected);

		testLog(' Show Page 5');
		$('#test BUTTON[idx]').eq(9).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.width'		, 351	, layers[0].buttons[0].r.width);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, true	, layers[0].buttons[2].r.selected);

		testLog(' Jump down');
		$('#test BUTTON[idx]').eq(10).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[13].r.selected'	, true	, layers[0].buttons[13].r.selected);

		testLog(' Jump down (2)');
		$('#test BUTTON[idx]').eq(11).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[11].r.selected'	, false	, layers[0].buttons[11].r.selected);
		test('layers[0].buttons[12].r.selected'	, false	, layers[0].buttons[12].r.selected);
		test('layers[0].buttons[13].r.selected'	, true	, layers[0].buttons[13].r.selected);

		testLog(' Move up');
		$('#test BUTTON[idx]').eq(12).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[11].r.selected'	, false	, layers[0].buttons[11].r.selected);
		test('layers[0].buttons[12].r.selected'	, true	, layers[0].buttons[12].r.selected);
		test('layers[0].buttons[13].r.selected'	, false	, layers[0].buttons[13].r.selected);

		testLog(' Move up (2)');
		$('#test BUTTON[idx]').eq(13).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[11].r.selected'	, true	, layers[0].buttons[11].r.selected);
		test('layers[0].buttons[12].r.selected'	, false	, layers[0].buttons[12].r.selected);
		test('layers[0].buttons[13].r.selected'	, false	, layers[0].buttons[13].r.selected);
		
		testLog(' Exit Filer');
		$('#test BUTTON[idx]').eq(14).click();
		test('layers.length'					, 0		, layers.length);

	});		

}

function test6() {
	
	// QuickPlay move selection from buttom to top

	return $.ajax({
		url : "test/quickplay-up-to-top.txt",
		dataType: "text",
	}).done(function (data) {
	
		testLog('\ntest QuickPlay move selection from buttom to top');
		
		decodeTestSequence(data);
		
		
		testLog(' Show QuickPlay (1)');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[8].r.selected'	, false	, layers[0].buttons[8].r.selected);
		test('layers[0].buttons[9].r.selected'	, true	, layers[0].buttons[9].r.selected);

		testLog(' Move up (1)');
		$('#test BUTTON[idx]').eq(1).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 10	, layers[0].buttons.length);
		test('layers[0].buttons[7].r.selected'	, false	, layers[0].buttons[7].r.selected);
		test('layers[0].buttons[8].r.selected'	, true	, layers[0].buttons[8].r.selected);
		test('layers[0].buttons[9].r.selected'	, false	, layers[0].buttons[9].r.selected);

		testLog(' Move up (2)');
		$('#test BUTTON[idx]').eq(2).click();
		test('layers[0].buttons[6].r.selected'	, false	, layers[0].buttons[6].r.selected);
		test('layers[0].buttons[7].r.selected'	, true	, layers[0].buttons[7].r.selected);
		test('layers[0].buttons[8].r.selected'	, false	, layers[0].buttons[8].r.selected);

		testLog(' Move up (3)');
		$('#test BUTTON[idx]').eq(3).click();
		test('layers[0].buttons[5].r.selected'	, false	, layers[0].buttons[5].r.selected);
		test('layers[0].buttons[6].r.selected'	, true	, layers[0].buttons[6].r.selected);
		test('layers[0].buttons[7].r.selected'	, false	, layers[0].buttons[7].r.selected);

		testLog(' Move up (4)');
		$('#test BUTTON[idx]').eq(4).click();
		test('layers[0].buttons[4].r.selected'	, false	, layers[0].buttons[4].r.selected);
		test('layers[0].buttons[5].r.selected'	, true	, layers[0].buttons[5].r.selected);
		test('layers[0].buttons[6].r.selected'	, false	, layers[0].buttons[6].r.selected);

		testLog(' Move up (5)');
		$('#test BUTTON[idx]').eq(5).click();
		test('layers[0].buttons[3].r.selected'	, false	, layers[0].buttons[3].r.selected);
		test('layers[0].buttons[4].r.selected'	, true	, layers[0].buttons[4].r.selected);
		test('layers[0].buttons[5].r.selected'	, false	, layers[0].buttons[5].r.selected);

		testLog(' Move up (6)');
		$('#test BUTTON[idx]').eq(6).click();
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);
		test('layers[0].buttons[3].r.selected'	, true	, layers[0].buttons[3].r.selected);
		test('layers[0].buttons[4].r.selected'	, false	, layers[0].buttons[4].r.selected);

		testLog(' Move up (7)');
		$('#test BUTTON[idx]').eq(7).click();
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, true	, layers[0].buttons[2].r.selected);
		test('layers[0].buttons[3].r.selected'	, false	, layers[0].buttons[3].r.selected);

		testLog(' Move up (8)');
		$('#test BUTTON[idx]').eq(8).click();
		test('layers[0].buttons[0].r.selected'	, false	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, true	, layers[0].buttons[1].r.selected);
		test('layers[0].buttons[2].r.selected'	, false	, layers[0].buttons[2].r.selected);

		testLog(' Move up (9)');
		$('#test BUTTON[idx]').eq(9).click();
		test('layers[0].buttons[0].r.selected'	, true	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[1].r.selected);
		
	});		
	
}

function test7() {

	// SmartEPG with page 5

	return $.ajax({
		url : "test/smartepg-page5.txt",
		dataType: "text",
	}).done(function (data) {

		testLog('\ntest SmartEPG with page 5');

		decodeTestSequence(data);


		testLog(' Show SmartEPG');
		$('#test BUTTON[idx]').eq(0).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.selected'	, true	, layers[0].buttons[0].r.selected);
		test('layers[0].buttons[1].r.selected'	, false	, layers[0].buttons[3].r.selected);

		testLog(' Show Page 5');
		$('#test BUTTON[idx]').eq(1).click();
		test('layers.length'					,  1	, layers.length);
		test('layers[0].buttons.length'			, 14	, layers[0].buttons.length);
		test('layers[0].buttons[0].r.width'		, 351	, layers[0].buttons[0].r.width);
		test('layers[0].buttons[0].r.selected'	, true	, layers[0].buttons[0].r.selected);

	});		

}

function runTests() {

	var deferred = $.Deferred();
	var pipe = deferred;

	// QuickPlay with delete dialog
	pipe = pipe.then(function() {
		return test1();
	});

	// FastSkip delete dialog
	pipe = pipe.then(function() {
		return test2();
	});

	// TMSArchive with delete dialog
	pipe = pipe.then(function() {
		return test3();
	});

	// Filer
	pipe = pipe.then(function() {
		return test4();
	});

	// SmartEPG with page 5 and navigation
	pipe = pipe.then(function() {
		return test5();
	});

	// QuickPlay move selection from buttom to top
	pipe = pipe.then(function() {
		return test6();
	});

	// SmartEPG with page 5
	pipe = pipe.then(function() {
		return test7();
	});

	deferred.resolve();
	
}
